/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.util.*;

public class Block implements java.io.Serializable
{
	private Vector sentenceList = new Vector();
	private boolean statsValid;
	private double cps;
	private double wpm;
	private double accuracy;
	private long whenStarted;

	public Block()
	{
		statsValid = false;
		whenStarted = System.currentTimeMillis();
	}

	public void addSentence(Stats sentence) {
		sentenceList.addElement(sentence);
		statsValid = false;
	}

	public long getTimeStarted() {
		return whenStarted;
	}

	public void getStats() {
		if (!statsValid) {
			double [] stats = Stats.getAggregateStats(sentenceList);
			cps = stats[0];
			wpm = stats[1];
			accuracy = stats[2];
			statsValid = true;
		}
	}

	public double getCPS() {
		getStats();
		return cps;
	}
	public double getWPM() {
		getStats();
		return wpm;
	}
	public double getAccuracy() {
		getStats();
		return accuracy;
	}

	public int size() {
		return sentenceList.size();
	}
	public Stats getSentence(int index) {
		return (Stats) sentenceList.elementAt(index);
	}
	public Vector getSentences() {
		return (Vector) sentenceList.clone();
	}
	public boolean sentenceUsed(String newSentence) {
		for (int i=0; i<size(); i++) {
			Stats sentence = getSentence(i);
			if (sentence.getSentence().equals(newSentence))
				return true;
		}
		return false;
	}
}
