/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import javax.swing.*;
import java.util.Vector;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * FingerPanel.java, a class for making the management of the finger pictures
 * easier. Close cousin to ThumbPanel.
 * 
 * Revisions:
 * 		0.5	17 July 2003
 * 		Completed Tutor
 * 		0.2	06 June 2003
 * 		Lots of bugfixes. Almost Final.
 * 		0.1	29 May 2003
 * 		Created class FingerPanel
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 17 July 2003
 */
public class FingerPanel extends JPanel implements TwiddlerSubPanel {

	/**
	 * The Array of JPanels that make up the buttons on our 'face'.
	 * For changing the background to highlighted color blah.
	 */
	private Vector myButtons;
	private TwidorConstants myDefaults;

	/**
	 * default constructor
	 * @param twidorDefaults the defaults to which we adhere
	 * @param keys the KeyMap we use to format our buttons
	 * @param int the finger number we are (1=index - 4=pinky)
	 */
	public FingerPanel (TwidorConstants twidorDefaults, KeyMap keys, int finger) {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		JPanel panel;
		Vector lookup;

		setDefaults(twidorDefaults);

		if (getDefaults().bDEBUG)
			System.out.println("FingerPanel: creating panel");
		setPreferredSize(new Dimension(getDefaults().TWIDDLER_X, (int)(getDefaults().WINDOW_Y / 5)));
		initButtons();

		/* How to arrange the keys) */
		if (getDefaults().TWIDDLER_MIRROR) {
			setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		}
		else {
			setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		}

		setLayout(gridbag); /* 6w x 4h grid */
		setBackground(getDefaults().TWIDDLER_BACKGROUND);

		constraints.fill = GridBagConstraints.BOTH;

		constraints.gridwidth = 1;
		constraints.gridheight = GridBagConstraints.RELATIVE;
		constraints.weightx = 1;
		constraints.weighty = 1;

		/* Row 1: Labels or Nothing */
		for (int i = 0; i < 6; i++) {
			panel = new JPanel();
			panel.setBackground(getDefaults().TWIDDLER_BACKGROUND);

			if (finger == 0) {
				if (i == 0) {
					panel.add(new JLabel(new ImageIcon(getDefaults().ICON_RED)));
				}
				else if (i == 2) {
					panel.add(new JLabel(new ImageIcon(getDefaults().ICON_BLUE)));
				}
				else if (i == 4) {
					panel.add(new JLabel(new ImageIcon(getDefaults().ICON_GREEN)));
				}
			}
			if (i == 5) {
				constraints.gridwidth = GridBagConstraints.REMAINDER;
			}

			gridbag.setConstraints(panel, constraints);
			add(panel);
		}

		constraints.gridwidth = 1;
		constraints.gridheight = GridBagConstraints.REMAINDER;
		constraints.weightx = 1;
		constraints.weighty = 3;

		for (int i = 0; i < 6; i++) {
			/*
			Vector buttons = new Vector(16,0);
			for (int x = 0; x < 16; x++) {
				buttons.add(x, new KeyStatus());
			}
			((KeyStatus)buttons.elementAt(finger * getDefaults().FINGER_OFFSET +
							      (int)(i / 2))).setStatus(true);
			*/
			KeyElement buttonsKey = new KeyElement();
			buttonsKey.setButton(finger * getDefaults().FINGER_OFFSET + (int)(i / 2), true);
			
			panel = new JPanel();
			if (i % 2 == 1) {
				/* Labels */
				panel.setBackground(getDefaults().TWIDDLER_BACKGROUND);
				panel.setLayout(new GridLayout(3, 1));
				if (i == 5) {
					constraints.gridwidth = GridBagConstraints.REMAINDER;
				}
				if (finger != 0 && getDefaults().TWIDDLER_SHOW_LETTERS) {
					for (int j = 0; j < 3; j++) {
						buttonsKey.setButton(getDefaults().INDEX_OFFSET + j, true);
						//((KeyStatus)buttons.elementAt(getDefaults().INDEX_OFFSET +
						//			      j)).setStatus(true);
						KeyElement myButton = keys.getKey(buttonsKey.getBits());
						if (myButton != null) {
							JLabel label;
							if (myButton.displayLetter().length() > 1) {
								label = new JLabel();
							}
							else {
								label = new JLabel(myButton.displayLetter().toUpperCase());
								label.setFont(new Font("Monospaced", Font.BOLD, 12));
								label.setHorizontalAlignment(JLabel.CENTER);
							}

							if (j == 0) {
								label.setForeground(Color.RED);
							}
							else if (j == 1) {
								label.setForeground(Color.BLUE);
							}
							else {
								label.setForeground(Color.GREEN);
							}
							panel.add(label);
						}
						//((KeyStatus)buttons.elementAt(getDefaults().INDEX_OFFSET +
						//		j)).setStatus(false);
						buttonsKey.setButton(getDefaults().INDEX_OFFSET + j, false);
					}
				}
				else {
					for (int j = 0; j < 3; j++) {
						panel.add(new JLabel());
					}
				}
			}
			else {
				/* Button */
				//panel.setPreferredSize(new Dimension(buttonX, buttonY));
				panel.setBackground(getDefaults().BUTTON_BACKGROUND);
				GridLayout layout = new GridLayout(3, 1);
				panel.setLayout(layout);
				panel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
				if (TwidorConstants.USE_MULTITAP) {
					JLabel label;
					if (finger == 0 && i == 0) {
						panel.add(quickLabel(""));
						panel.add(quickLabel("S"));
						panel.add(quickLabel("P"));
					} else if (finger == 0 && i == 2) {
						panel.add(quickLabel("A"));
						panel.add(quickLabel("B"));
						panel.add(quickLabel("C"));
					} else if (finger == 0 && i == 4) {
						panel.add(quickLabel("D"));
						panel.add(quickLabel("E"));
						panel.add(quickLabel("F"));
					} else if (finger == 1 && i == 0) {
						panel.add(quickLabel("G"));
						panel.add(quickLabel("H"));
						panel.add(quickLabel("I"));
					} else if (finger == 1 && i == 2) {
						panel.add(quickLabel("J"));
						panel.add(quickLabel("K"));
						panel.add(quickLabel("L"));
					} else if (finger == 1 && i == 4) {
						panel.add(quickLabel("M"));
						panel.add(quickLabel("N"));
						panel.add(quickLabel("O"));
					} else if (finger == 2 && i == 0) {
						panel.setLayout(new GridLayout(4, 1));
						panel.add(quickLabel("P"));
						panel.add(quickLabel("Q"));
						panel.add(quickLabel("R"));
						panel.add(quickLabel("S"));
						//layout.setVgap(-10);
					} else if (finger == 2 && i == 2) {
						panel.add(quickLabel("T"));
						panel.add(quickLabel("U"));
						panel.add(quickLabel("V"));
					} else if (finger == 2 && i == 4) {
						panel.setLayout(new GridLayout(4, 1));
						panel.add(quickLabel("W"));
						panel.add(quickLabel("X"));
						panel.add(quickLabel("Y"));
						panel.add(quickLabel("Z"));
					} else if (finger == 3 && i == 0) {
						panel.add(quickLabel(""));
						panel.add(quickLabel("<"));
					} else if (finger == 3 && i == 2) {
						panel.add(quickLabel("E"));
						panel.add(quickLabel("N"));
						panel.add(quickLabel("T"));
					} else if (finger == 3 && i == 4) {
						panel.add(quickLabel(""));
						panel.add(quickLabel(">"));
					} else {
						panel.add(quickLabel("0"));
					}
				} else {
					/* Button */
					//panel.setPreferredSize(new Dimension(buttonX, buttonY));

					KeyElement myButton = keys.getKey(buttonsKey.getBits());
					if (myButton != null && getDefaults().TWIDDLER_SHOW_LETTERS) {
						JLabel label;
						if (myButton.getNumber() == getDefaults().KEY_SPACE) {
							// space
							panel.add(new JLabel());
							label = new JLabel("S");
							label.setFont(new Font("Monospaced", Font.BOLD, 12));
							label.setHorizontalAlignment(JLabel.CENTER);
							panel.add(label);
							label = new JLabel("P");
						}
						else if (myButton.getNumber() == getDefaults().KEY_BACKSPACE) {
							// backspace
							panel.add(new JLabel());
							label = new JLabel("B");
							label.setFont(new Font("Monospaced", Font.BOLD, 12));
							label.setHorizontalAlignment(JLabel.CENTER);
							panel.add(label);
							label = new JLabel("S");
						}
						else if (myButton.getNumber() == getDefaults().KEY_DELETE) {
							// delete
							label = new JLabel("D");
							label.setFont(new Font("Monospaced", Font.BOLD, 12));
							label.setHorizontalAlignment(JLabel.CENTER);
							panel.add(label);
							label = new JLabel("E");
							label.setFont(new Font("Monospaced", Font.BOLD, 12));
							label.setHorizontalAlignment(JLabel.CENTER);
							panel.add(label);
							label = new JLabel("L");
						}
						else if (myButton.getNumber() == getDefaults().KEY_ENTER) {
							// enter
							label = new JLabel("E");
							label.setFont(new Font("Monospaced", Font.BOLD, 12));
							label.setHorizontalAlignment(JLabel.CENTER);
							panel.add(label);
							label = new JLabel("N");
							label.setFont(new Font("Monospaced", Font.BOLD, 12));
							label.setHorizontalAlignment(JLabel.CENTER);
							panel.add(label);
							label = new JLabel("T");
						}
						else {
							panel.add(new JLabel());
							label = new JLabel(myButton.displayLetter().toUpperCase());
						}
						label.setFont(new Font("Monospaced", Font.BOLD, 12));
						label.setHorizontalAlignment(JLabel.CENTER);
						panel.add(label);
					}
					else {
						JLabel label = new JLabel();
						panel.add(label);
					}
				}
			}
			gridbag.setConstraints(panel, constraints);
			if (i % 2 == 0) {
				/* We want to keep the "Buttons" around to 'highlight' */
				getButtons().add(panel);
			}
			add(panel);
		}

		if (getDefaults().bDEBUG)
			System.out.println("FingerPanel: panel created");
	}// end FingerPanel (boolean, KeyMap)

	private static JLabel quickLabel(String s) {
		JLabel label = new JLabel(s);
		label.setFont(new Font("Monospaced", Font.BOLD, 12));
		label.setHorizontalAlignment(JLabel.CENTER);
		label.setPreferredSize(new Dimension(5, 5));
		return label;
	}

	/**
	 * Modifier for internal variable
	 * @param twidorDefaults our TwidorConstants
	 */
	private void setDefaults (TwidorConstants twidorDefaults) {
		myDefaults = twidorDefaults;
	}// end setDefaults (TwidorConstants)

	/**
	 * accessor for internal variable
	 * @return TwidorConstants the twiddler constants
	 */
	private TwidorConstants getDefaults () {
		return myDefaults;
	}// end getDefaults ()

	/**
	 * Quick internal accessor since I was confusing myself by accessing it directly
	 * @return Vector my button
	 */
	public Vector getButtons () {
		return myButtons;
	}// end getMyButton ()

	/**
	 * Quick internal accessor to initialize my buttons
	 */
	private void initButtons () {
		myButtons = new Vector();
	}// end setButtons ()


	/* --==<( TwiddlerSubPanel Requirement )>==-- */
	/**
	 * method for highlighting a certain button to a certain color
	 * @param which which button we need to highlight (0-2)
	 * @param hColor what color you want it to become
	 */
	public void highlight (int which, Color hColor) {
		if (getDefaults().bDEBUG)
			System.out.println("FingerPanel: highlight button " + which);
		try {
			((JPanel)getButtons().elementAt(which)).setBackground(hColor);
		}
		catch (ArrayIndexOutOfBoundsException e) {
			if (getDefaults().bDEBUG)
				System.out.println("FingerPanel: array oob");
		}
	}// end highlight (Color)

	/* --==<( TwiddlerSubPanel Requirement )>==-- */
	/**
	 * method for clearing the background of the buttons
	 */
	public void clear () {
		for (int i = 0; i < getButtons().size(); i++) {
			((JPanel)getButtons().elementAt(i)).setBackground(getDefaults().BUTTON_BACKGROUND);
		}
	}// end clear ()

}// end class FingerPanel
