/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.util.Vector;
import java.lang.Object;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * KeyElement.java, a class for making matching keys to letters sooooooo much
 * easier.
 * 
 * Revisions:
 * 		0.5	17 July 2003
 * 		Completed Tutor
 * 		0.2	06 June 2003
 * 		Bugs Stomped. Final Version.
 * 		0.1	23 May 2003
 * 		Created class KeyElement
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 06 July 2003
 */

import java.io.Serializable;

public class KeyElement extends Object implements Cloneable, Serializable {

	/**
	 * the button map and the letter we represent
	 */
	private String letter;
	private int number;
	//private Vector buttons;
	private int buttonStatus = -1;
	private long time;

	public static final int NUM_BUTTONS = 16;

	/**
	 * default constructor with no arguments
	 */
	public KeyElement () {
		/*
		 * Vector buttonlist = new Vector();
		for (int i = 0; i < 16; i++) {
			buttonlist.add(i, new KeyStatus());
		}
		setButtons(buttonlist);
		*/
		buttonStatus = 0;
		setLetter(null);
		setTime(System.currentTimeMillis());
	}// end KeyElement

	/**
	 * default constructor with the integer value of the key we want
	 * @param thing the integer value of the key
	 */
	public KeyElement (int thing) {
		this();
		setNumber(thing);
	}// end KeyElement (int)

	public KeyElement (int thing, long time) {
		this();
		setNumber(thing);
		setTime(time);
	}

	/**
	 * default constructor with the letter/macro value of the key we want
	 * @param thing the letter/macro to set
	 */
	public KeyElement (String thing) {
		this();
		setLetter(thing);
	}// end KeyElement (String)
	
	/**
	 * default constructor with a premade KeyElement to copy
	 * @param toCopy the key to duplicate
	 */
	public KeyElement (KeyElement toCopy) {
		setTime(toCopy.getTime());
		//setButtons(toCopy.getButtons());
		buttonStatus = toCopy.buttonStatus;
		if (toCopy.getLetter() == null) {
			setNumber(toCopy.getNumber());
		}
		else {
			setLetter(toCopy.getLetter());
		}
	}// end KeyElement (KeyElement)

	/**
	 * get the time this key was pressed
	 * @return long the time in ms of pressing
	 */
	public long getTime () {
		return time;
	}// end getTime ()

	/**
	 * set the time this key was pressed
	 * @param thing the time of pressing in milliseconds
	 */
	public void setTime (long thing) {
		time = thing;
	}// end setTime (long)

	/**
	 * get the letter or macro this KeyElement holds.
	 * @return String the letter this Element holds
	 */
	public String getLetter () {
		return letter;
	}// end getLetter ()

	/**
	 * set the letter or macro of this KeyElement.
	 * @param thing the new letter/macro for this key
	 */
	public void setLetter (String thing) {
		letter = thing;
		number = -1;
	}// end setLetter (string)

	/**
	 * accessor for the numeric value of this key
	 * @return int the numeric value
	 */
	public int getNumber () {
		return number;
	}// end getNumber ()

	/**
	 * modifier for the numeric value of this key
	 * @param that the numeric value of the key we represent
	 */
	public void setNumber (int that) {
		number = that;
		letter = null;
	}// end setNumber (int)

	/**
	 * accessor for the value of this key
	 * @return char the character this number matches
	 */
	public String displayLetter () {
		if (getNumber() != -1) {
			return String.valueOf((char)getNumber());
		}
		return getLetter();
	}// end getChar ()

	/**
	 * get a specific button's state
	 * @param button the button we want the state of
	 * @return boolean the status of the button
	 */
	public boolean getButton (int button) {
		//return ((KeyStatus)getButtons().elementAt(button)).getStatus();

		return ((buttonStatus & (1 << button)) != 0);

	}// end getButton (int)

	/**
	 * get a whole bunch of buttons
	 * @return Vector the list of buttons
	 */

	//	0 is index finger, 1 is ring finger, etc
	public int getFingerPosition(int finger) {
		int base = finger * 3;
		if (this.getButton(base)) return TwidorConstants.B_LEFT;
		if (this.getButton(base + 1)) return TwidorConstants.B_MIDDLE;
		if (this.getButton(base + 2)) return TwidorConstants.B_RIGHT;
		return TwidorConstants.B_NONE;
	}
	
	public int getBits() {
		return buttonStatus;
	}
	
	public void setBits(int bits) {
		buttonStatus = bits;
	}

	/**
	 * set the button status
	 * @param button the button to modify the state of
	 * @param status the new state to set
	 */
	public void setButton (int button, boolean status) {
		//setButton(button, new KeyStatus(status));

		int bit = 1 << button;
		if (status) buttonStatus |= bit;
		else buttonStatus &= ~bit;

	}// end setButton (int, boolean)

	/**
	 * set the button status
	 * @param button the button to modify the state of
	 * @param status the new encapsulated state
	 */
	public void setButton (int button, KeyStatus status) {
		//getButtons().setElementAt(status, button);
		setButton(button, status.getStatus());
	}// end setButton (int, KeyStatus)

	/**
	 * set the button status
	 * @param buttonlist the vector of button states this key has
	 */
	/*
	public void setButtons (Vector buttonlist) {
		buttonStatus = 0;
		for(int i=0; i<16; i++) {
			KeyStatus status = (KeyStatus) buttonlist.elementAt(i);
			int bit = status.getStatus() ? 1 : 0;
			buttonStatus |= (bit << i);
		}
	}// end setButtons (Vector)
	*/

	/**
	 * default comparator
	 * @param o the object to compare it to
	 * @return boolean true if equals, false otherwise
	 */
	public boolean equals (Object o) {
		if (o instanceof KeyElement) {
			KeyElement temp = (KeyElement)o;

			if ((temp.displayLetter() != null) && (displayLetter() != null)) {
				if (temp.displayLetter().equals(displayLetter())) return true;
			}
			if (temp.buttonStatus == buttonStatus) return true;
			/*
			if ((temp.getButtons() != null) && (getButtons() != null)) {
				for (int i = 0; i < getButtons().size(); i++) {
					if (getButton(i) != temp.getButton(i)) {
						return false;
					}
				}
				return true;
			}
			*/
		}
		return false;
	}// end equals (Object)

	public int hashCode() {
		return buttonStatus;
	}

	/**
	 * Function for returning this KeyElement in an easy-to-debug format
	 * @return String the text version of this KeyElement
	 */
	public String toString () {
		String toReturn = "";
		toReturn += time + "\t";
		
		if (getNumber() != -1) {
			toReturn += (getNumber() + "\t");
		} else {
			for (int i=0; i<getLetter().length(); i++) {
				if (i != 0) toReturn += ",";
				toReturn += (int) getLetter().charAt(i);
			}
			toReturn += "\t";
		}
		
		for (int i = 0; i < 16; i++) {
			if (getButton(i)) {
				toReturn += "1 ";
			}
			else {
				toReturn += "0 ";
			}
		}
		return toReturn;
	}// end toString ()
	
	/**
	 * Function that takes the output of the toString()
	 * method and converts it back into a KeyElement object
	 */
	public static KeyElement fromString(String string) {
		KeyElement ret = new KeyElement();
		String [] words = string.split(" |\t");
		
		try {
			ret.time = Long.parseLong(words[0]);
			String [] numbers = words[1].split(",");
			if (numbers.length == 1) {
				ret.number = Integer.parseInt(words[1]);
			} else {
				ret.letter = "";
				for (int i=0; i< numbers.length; i++) {
					ret.letter += (char) Integer.parseInt(numbers[i]);
				}
			}
			
			for (int i=0; i<16; i++) {
				ret.setButton(i, words[i+2].equals("1"));
				//ret.buttons.add(i, new KeyStatus(words[i+2].equals("1")));
			}
			
		} catch (NumberFormatException e) {
			if (TwidorConstants.bDEBUG)
				System.out.println("KeyElement.fromString: bad number format");
			return null;
		} catch (IndexOutOfBoundsException e) {
			if (TwidorConstants.bDEBUG)
				System.out.println("KeyElement.fromString: not enough items");
			return null;
		}
		
		return ret;

	}

	/**
	 * Overrides the default Object.clone()
	 */
	public Object clone () {
		return (new KeyElement(this));
	}// end clone ()

	public static class KeyPair {
		public KeyElement k1;
		public KeyElement k2;
		public KeyPair(KeyElement k1, KeyElement k2) {
			this.k1 = k1;
			this.k2 = k2;
		}
		public boolean equals(Object o) {
			if (!(o instanceof KeyPair)) return false;
			KeyPair other = (KeyPair) o;
			return (k1.equals(other.k1) && k2.equals(other.k2));
		}
		public int hashCode() {
			return (k1.hashCode() << 16) + k2.hashCode();
		}
	}

}// end class KeyElement
