/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.util.*;

public class LanguageModel
{
	char [] chars;
	int [][] frequencies;
	int totalPairs;

	public LanguageModel()
	{
	}

	public double predictWPM(TypingModel typingModel) {
		double runningTotal = 0;

		for (int firstChar=0; firstChar<chars.length; firstChar++) {
			for (int secondChar=0; secondChar<chars.length; secondChar++) {
				double prob = frequencies[firstChar][secondChar] / (double) totalPairs;
				double delay = typingModel.getDelay(chars[firstChar], chars[secondChar]);

				//	Are we going to run into precision errors here?
				runningTotal += prob * delay;
			}
		}

		double CPS = 1/runningTotal;

		double WPM = CPS * 60 / 5;

		return WPM;
	}

	public void read(String filename) {
		try {
			BufferedReader tableReader = new BufferedReader(new FileReader(filename));

			String line = tableReader.readLine();
			String [] firstline = line.split("\t");

			ArrayList charList = new ArrayList();
			for (int i=1; i<firstline.length; i++) {
				if (firstline[i].equalsIgnoreCase("space")) {
					charList.add(" ");
				} else if (!firstline[i].equals("")) {
					charList.add(String.valueOf(Character.toLowerCase(firstline[i].charAt(0))));
				}
			}

			chars = new char[charList.size()];
			frequencies = new int[charList.size()][charList.size()];

			for (int i=0; i<chars.length; i++) {
				String ch = (String) charList.get(i);
				chars[i] = ch.charAt(0);
			}

			totalPairs = 0;
			for (int firstChar = 0; firstChar < chars.length; firstChar++) {
				line = tableReader.readLine();
				String [] items = line.split("\t");

				for (int secondChar = 0; secondChar < chars.length; secondChar++) {
					int frequency = Integer.parseInt(items[secondChar + 1]);
					frequencies[firstChar][secondChar] = frequency;
					totalPairs += frequency;
				}
			}

			tableReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (NumberFormatException e) {
			e.printStackTrace();
		}
	}
}
