/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import javax.swing.*;
import java.util.Vector;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * MultiTapPanel.java, a class for displaying a multi-tap keyboard layout.
 *
 * Revisions:
 * 		0.1	01 September 2003
 * 			Created class MultiTapPanel
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.1; 01 September 2003
 */
public class MultiTapPanel extends JPanel implements TwiddlerSubPanel {

	/**
	 * internal variables
	 */
	private Vector myButtons;
	private TwidorConstants myDefaults;

	/**
	 * default constructor
	 */
	private MultiTapPanel () {
	}// end MultiTapPanel ()

	/**
	 * default constructor
	 */
	public MultiTapPanel (TwidorConstants twidorDefaults, KeyMap keys, int row) {
		setDefaults(twidorDefaults);
		myButtons = new Vector();

		if (getDefaults().bDEBUG)
			System.out.println("MultiTapPanel: creating panel");

		/* set the panel defaults */
		setPreferredSize(new Dimension(getDefaults().TWIDDLER_X,
					(int)(getDefaults().WINDOW_Y / 4)));
		if (getDefaults().TWIDDLER_MIRROR) {
			setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		}
		else {
			setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		}
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		setBackground(getDefaults().TWIDDLER_BACKGROUND);

		/* start laying out the buttons */
		add(Box.createHorizontalGlue());
		for (int i = 0; i < 3; i++) {
			add(makeButton(row, i));
			add(Box.createHorizontalGlue());
		}
	}// end MultiTapPanel (TwidorConstants, KeyMap, int)

	/**
	 * function to add a Button set to the main panel
	 * @param int the row we're adding
	 * @param int the button we're adding
	 * @return JPanel the completed button set
	 */
	private JPanel makeButton (int row, int column) {
		int button = row * 3 + column + 1;
		JPanel toReturn = new JPanel();
		toReturn.setBackground(getDefaults().TWIDDLER_BACKGROUND);
		if (getDefaults().TWIDDLER_MIRROR) {
			toReturn.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		}
		else {
			toReturn.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		}
		toReturn.setPreferredSize(new Dimension((int)(getDefaults().TWIDDLER_X / 3), 
					(int)(getDefaults().WINDOW_Y / 4)));

		setLayout(new BoxLayout(toReturn, BoxLayout.LINE_AXIS));
		/* FIXME */
		/* the left panel! */
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		left.setAlignmentX(CENTER_ALIGNMENT);
		left.setAlignmentY(CENTER_ALIGNMENT);
		switch (button) {
			case 2:		// 2
			case 3:		// 3
			case 4:		// 4
			case 5:		// 5
			case 6:		// 6
				for (int i = 0; i < 3; i++) {
					left.add(quickLabel(
						String.valueOf((char)(97 + (button - 2) * 3 + i))));
				}
				break;
			case 7:		// 7
				for (int i = 0; i < 4; i++) {
					left.add(quickLabel(String.valueOf((char)(112 + i))));
				}
				break;
			case 8:		// 8
				for (int i = 0; i < 3; i++) {
					left.add(quickLabel(String.valueOf((char)(116 + i))));
				}
				break;
			case 9:		// 9
				for (int i = 0; i < 4; i++) {
					left.add(quickLabel(String.valueOf((char)(119 + i))));
				}
				break;
			case 10:	// *
			case 12:	// #
				break;
			default:
				System.out.println("Bug! " + button);
				break;
		}

		/* the right panel! */
		JPanel right = new JPanel();
		right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
		right.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		right.setPreferredSize(new Dimension(getDefaults().BUTTON_X, getDefaults().BUTTON_Y));
		right.setAlignmentX(CENTER_ALIGNMENT);
		right.setAlignmentY(CENTER_ALIGNMENT);
		if (button <= 9) {
			right.add(quickLabel(String.valueOf(button)));
			right.add(quickLabel(String.valueOf(button + 1)));
		}
		else if (button == 10) {
			right.add(quickLabel("*"));
		}
		else if (button == 11) {
			right.add(quickLabel(String.valueOf(0)));
		}
		else if (button == 12) {
			right.add(quickLabel("#"));
		}
		else {
			System.out.println("Bug! " + button);
		}

		toReturn.add(Box.createHorizontalGlue());
		toReturn.add(left);
		toReturn.add(right);
		toReturn.add(Box.createHorizontalGlue());

		return toReturn;
	}// end makeButton (int, int)

	/**
	 * function to make a quick JLabel
	 * @param String the letter to put on the face
	 */
	private JLabel quickLabel (String bleh) {
		JLabel toReturn = new JLabel(bleh);
		toReturn.setFont(new Font("Monospaced", Font.PLAIN, 12));
		toReturn.setForeground(getDefaults().TEXT_DEFAULT);
		toReturn.setBackground(getDefaults().TEXT_BACKGROUND);
		toReturn.setHorizontalAlignment(SwingConstants.CENTER);
		return toReturn;
	}

	/**
	 * modifier
	 * @param TwidorConstants the constants to set
	 */
	private void setDefaults (TwidorConstants twidorDefaults) {
		myDefaults = twidorDefaults;
	}// end setDefaults (TwidorConstants)

	/**
	 * accessor
	 * @return TwidorConstants the constants for this panel
	 */
	private TwidorConstants getDefaults() {
		return myDefaults;
	}// end getDefaults()

	/**
	 * accessor
	 * @return Vector the list of buttons this panel holds
	 */
	private Vector getButtons () {
		return myButtons;
	}// end getButtons ()

// TwidorSubPanel requirements
	/**
	 * a function for clearing all of the buttons
	 */
	public void clear () {
		for (int i = 0; i < getButtons().size(); i++) {
			((JPanel)getButtons().elementAt(i)).setBackground(
					getDefaults().BUTTON_BACKGROUND);
		}
	}// end clear ()

	/**
	 * a function for highlighting only a certain button
	 * @param int the button to highlight
	 * @param Color the color to hightlight it to
	 */
	public void highlight (int which, Color hColor) {
		/* FIXME: treat 'which' as a key.getNumber() */
		if (getDefaults().bDEBUG)
			System.out.println("MultiTapPanel: highlight button " + which);
		try {
			((JPanel)getButtons().elementAt(which)).setBackground(hColor);
		} catch (ArrayIndexOutOfBoundsException e) {
			if (getDefaults().bDEBUG)
				System.out.println("MultiTapPanel: Array OOB");
		}
	}// end highlight (int, Color)

}// end class MultiTapPanel
