/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.io.*;
import java.util.*;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * Sentence.java, the Sentence Reader/Storing array.
 *
 * Revisions:
 * 			0.5	17 July 2003
 * 			Completed Tutor
 * 			0.1	22 May 2003
 * 			Created class Sentence
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 17 July 2003
 */
public class Sentence {

	/**
	 * The internal Sentence Vector
	 */
	private Vector mySentences;
	private TwidorConstants myDefaults;

	/**
	 * default constructor
	 */
	private Sentence () {
	}// end Sentence

	/**
	 * default constructor
	 * @param String the file to open
	 */
	public Sentence (TwidorConstants twidorDefaults) {
		mySentences = new Vector();
		myDefaults = twidorDefaults;
		readFile(getDefaults().DEFAULT_PHRASES);
		getDefaults().SENTENCE_NUMBER = 0;
	}// end Sentence (String)

	/**
	 * default accessor
	 * @return TwidorConstants the constants for this file
	 */
	private TwidorConstants getDefaults () {
		return myDefaults;
	}// end getDefaults ()

	/**
	 * Returns the Sentence at the specified position.
	 * @param int the requested sentence
	 * @return String the requested sentence or null
	 */
	public String getSentence (int which) {

		if (which == TwidorConstants.SENTENCE_ALPHA1)
			return "abcd efgh ijkl";
		else if (which == TwidorConstants.SENTENCE_ALPHA2)
			return "mnop qrst uvwx yz";

		if (which < 0 || which > getSize()) {
			return null;
		}
		return (String)mySentences.elementAt(which);
	}// end getSentence (int)

	/**
	 * Returns the number of Sentences available.
	 * @return int the number of Sentences
	 */
	public int getSize () {
		return mySentences.size();
	}// end getSize ()

	/**
	 * Adds the specified Sentence to those available.
	 * @param String the sentence to add
	 */
	private void addSentence (String what) {
		mySentences.add(what);
	}// end setSentence (String)

	/**
	 * Opens the specified file and reads from it one line at a time until
	 * they are all stored in their own Sentence vectors.
	 * @param String the file to open
	 */
	public void readFile (String source) {
		InputStream iStream;
		InputStreamReader isReader;
		BufferedReader newBufferedReader;

		if (getDefaults().bDEBUG)
			System.out.println("Sentence: reading file " + source);

		try {
			/* FIXME: support reading from both a jar and a fs file */
			iStream = this.getClass().getResourceAsStream(source);
			isReader = new InputStreamReader(iStream);
			newBufferedReader = new BufferedReader(isReader);
			while (newBufferedReader.ready()) {
				addSentence(newBufferedReader.readLine());
			}
			newBufferedReader.close();
			isReader.close();
			iStream.close();
		} catch (NullPointerException e) {
			System.out.println("Sentence File not found: " + source);
		} catch (IOException e) {
			if (getDefaults().bDEBUG)
				System.out.println("Sentence: IO Error while reading " + source);
		} catch (Exception e) {
			if (getDefaults().bDEBUG)
				System.out.println("Sentence: General Error: " + source);
		}

		if (getDefaults().bDEBUG)
			System.out.println("Sentence: " + source + " read. " +
					getSize() + " sentences.");
	}// end readFile (String)

}// end class Sentence
