/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.util.Vector;
import java.util.*;
import java.io.*;
import java.text.DecimalFormat;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * Stats.java, what I expect to be the most heavily modified file here.
 * This file is meant to hold the stats for a user/session/whatever. For
 * however long. And automatically do the calculations. Etc.
 *
 * Revisions:
 * 			0.5	17 July 2003
 * 			Completed Tutor
 * 			0.1	22 May 2003
 * 			Created class Stats
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 17 July 2003
 */
import java.io.Serializable;

public class Stats implements Serializable {

	/**
	 * internal variable that contains stats
	 */
	private String sentence;
	private Vector statArray;
	private int keys;
	private int states;
	private int inputType;
	
	/**
	 * default constructor
	 */
	private Stats () {
	}// end Stats

	/**
	 * default constructor
	 */
	public Stats (String newSentence, int inputType) {
		this.inputType = inputType;
		setSentence(newSentence);
		setKeyPresses(new Vector());
		setKeyCount(0);
		setStateCount(0);
	}// end Stats (String)

	/**
	 * Accessor to get the sentence for this stat
	 * @return String the sentence we contain
	 */
	public String getSentence () {
		return sentence;
	}// end getSentence ()

	/**
	 * Accessor to get the vector of keypresses
	 * @return Vector of StatsElement
	 */
	public Vector getKeyPresses () {
		return statArray;
	}// end getKeyPresses ()

	/**
	 * Accessor for getting the number of keys
	 * @return int the number of keys
	 */
	public int getKeyCount () {
		return keys;
	}// end getKeyCount ()

	/**
	 * Accessor for getting the number of states
	 * @return int the number of states
	 */
	public int getStateCount () {
		return states;
	}// end getStateCount ()

	/**
	 * Modifier to set the current sentence
	 * @param String the sentence to set
	 */
	public void setSentence (String line) {
		sentence = line;
	}// end setSentece (String)

	/**
	 * Modifier to set the KeyPress vector
	 * @param Vector the new vector to set
	 */
	public void setKeyPresses (Vector poke) {
		statArray = poke;
	}// end setKeyPresses (Vector)

	/**
	 * Modifier to set the number of actual Keys contained
	 * @param int the new number
	 */
	public void setKeyCount (int number) {
		keys = number;
	}// end setKeyCount (int)
	
	/**
	 * Modifier to set the number of actual states contained
	 * @param int the number of states
	 */
	public void setStateCount (int number) {
		states = number;
	}// end setStateCount (int)

	public int getInputType() {
		return inputType;
	}
	
	/**
	 * Accessor to add a new StatsElement to the Stats
	 * @param KeyElement the key that was pressed
	 * @param long the time it took to press it (in ms)
	 */
	public void addPressed (KeyElement key) {
		if (key.getLetter() == null) {
			setKeyCount(getKeyCount() + 1);
			getKeyPresses().addElement(key.clone());
		} else {
			for (int i=0; i<key.getLetter().length(); i++) {
				KeyElement newKey = new KeyElement((int)key.getLetter().charAt(i), key.getTime());
				newKey.setBits(key.getBits());
				addPressed(newKey);
			}
		}
	}// end addKey (KeyElement, KeyElement, long)

	/**
	 * Accessor to add a new StatsElement to the Stats
	 * @param KeyElement the new key state
	 */
	public void addState (KeyElement key) {
		setStateCount(getStateCount() + 1);
		getKeyPresses().addElement(key.clone());
	}// end addState (KeyElement)


	/**
	 * Converts an ArrayList of Integers into an array of ints
	 */
	private int [] getIntArray(ArrayList list) {
		if (list.size() == 0) return null;

		int [] array = new int[list.size()];
		for (int i = 0; i < list.size(); i++) {
			array[i] = ((Integer) list.get(i)).intValue();
		}
		return array;
	}// end getIntArray(ArrayList)

	/**
	 * Converts a String into an array of ints.
	 */
	private int [] getIntArray(String string) {
		if (string.length() == 0) return null;

		int [] array = new int[string.length()];
		for (int i = 0; i < string.length(); i++) {
			//array[i] = new Character(string.charAt(i)).getNumericValue();
			array[i] = string.charAt(i);
		}

		return array;
	}// end getIntArray(String)

	/**
	 * Converts an array of ints to a string.
	 * Interprets each integer as a character code.
	 */
	private String intArrayToString(int [] array) {
		String ret = "";
		for (int i = 0; i < array.length; i++) {
			ret += (char) array[i];
		}
		return ret;
	}// end intArrayToString(int [])

	public ArrayList [] getTransitionDelays(KeyElement k1, KeyElement k2) {
		Map delays = getTransitionDelays();
		KeyElement.KeyPair key = new KeyElement.KeyPair(k1, k2);
		ArrayList [] ret = (ArrayList []) delays.get(key);
		if (ret == null) {
			ret = new ArrayList[3];
			for (int i=0; i<3; i++) {
				ret[i] = new ArrayList();
			}
		}
		return ret;
	}
	public Map getTransitionDelays() {
		HashMap map = new HashMap();
		getTransitionDelays(map);
		return map;
	}
	public void getTransitionDelays(Map map) {
		KeyElement key = new KeyElement();
		KeyElement lastKey = new KeyElement();

		boolean needKey = true;

		KeyElement lastKeyPressed = null;
		KeyElement lastKeyReleased = null;
		KeyElement keyPressed = null;
		KeyElement keyReleased = null;

		Iterator it = getKeyPresses().iterator();
		while (it.hasNext()) {
			key = (KeyElement) it.next();

			int changed = key.getBits() ^ lastKey.getBits();

			if (changed != 0 && (lastKey.getBits() & changed) != 0) {
				//	Released something

				if (keyPressed != null) {
					keyReleased = key;

					if (lastKeyPressed != null) {
						//System.out.println("Key input: " + keyReleased);

						KeyElement.KeyPair keyPair = new KeyElement.KeyPair(lastKeyPressed, keyPressed);
						ArrayList [] timings = (ArrayList []) map.get(keyPair);
						if (timings == null) {
							timings = new ArrayList[3];
							for (int i=0; i<3; i++) {
								timings[i] = new ArrayList();
							}
							map.put(keyPair, timings);
						}

						timings[0].add(new Long(lastKeyReleased.getTime() - lastKeyPressed.getTime()));
						timings[1].add(new Long(keyPressed.getTime() - lastKeyReleased.getTime()));
						timings[2].add(new Long(keyReleased.getTime() - keyPressed.getTime()));
					}

					lastKeyPressed = keyPressed;
					lastKeyReleased = keyReleased;

					keyPressed = null;
					keyReleased = null;
				}
			}

			if (changed != 0 && (key.getBits() & changed) != 0) {
				//	Pressed something
				keyPressed = key;
			}
			lastKey = key;
		}
	}


	/**
	 * Determines whether two keycodes map to the same physical
	 * multitap key.
	 */
	public static boolean sameKey(int key1, int key2) {
		if (key1 == key2) return true;
		
		int diff = key1 - key2;
		if (diff > 3 || diff < -3) return false;
		
		if (inRange(key1,key2, 103, 105)) return true;
		if (inRange(key1,key2, 112, 115)) return true;
		if (inRange(key1,key2, 97, 99)) return true;
		if (inRange(key1,key2, 106, 108)) return true;
		if (inRange(key1,key2, 116, 118)) return true;
		if (inRange(key1,key2, 100, 102)) return true;
		if (inRange(key1,key2, 109, 111)) return true;
		if (inRange(key1,key2, 119, 122)) return true;
		
		return false;
	}

	/*
0     L000 = 32		# 1
0     0L00 = 103	# 4ghi
0     00L0 = 112	# 7pqrs
0     000L = 42		# *<
0     M000 = 97		# 2abc
0     0M00 = 106	# 5jkl
0     00M0 = 116	# 8tuv
0     000M = 48		# 0
0     R000 = 100	# 3def
0     0R00 = 109	# 6mno
0     00R0 = 119	# 9wxyz
0     000R = 35		# #>

	
	*/
	
	private static boolean inRange(int key1, int key2, int lower, int upper) {
		return (lower <= key1 && lower <= key2 && key1 <= upper && key2 <= upper);
	}

	public static class SentenceModification {
		public KeyElement keyElement;
		public String string;
		public int action;

		public static final int ACTION_NONE = 0;
		public static final int ACTION_APPEND = 1;
		public static final int ACTION_APPEND_ADVANCE = 2;
		public static final int ACTION_REPLACE = 3;
		public static final int ACTION_ADVANCE = 4;
		public static final int ACTION_BACKSPACE = 5;
	}

	public static SentenceModification parseMultiTapLetter(KeyElement key, char cursor) {
		SentenceModification ret = new SentenceModification();
		ret.keyElement = new KeyElement(key);
		ret.string = "";


		int code = key.getNumber();
		if (code == TwidorConstants.KEY_NOT_A_KEY) {
			ret.action = SentenceModification.ACTION_NONE;
		} else if (code == TwidorConstants.KEY_MULTITAP || code == TwidorConstants.KEY_RIGHT) {
			if (cursor == 0) {
				//	Nothing has been typed at current cursor location.  So we
				//	insert a space and advance
				ret.action = SentenceModification.ACTION_APPEND_ADVANCE;
				//ret.keyElement.setNumber(TwidorConstants.KEY_SPACE);
				ret.string = " ";
			} else {
				ret.action = SentenceModification.ACTION_ADVANCE;
			}
		} else if (code == TwidorConstants.KEY_LEFT) {
			ret.action = SentenceModification.ACTION_BACKSPACE;
		} else if (code == TwidorConstants.KEY_SPACE) {
			ret.action = SentenceModification.ACTION_APPEND_ADVANCE;
			ret.string = " ";
		} else if (code > 0 && code != TwidorConstants.KEY_MENTER) {
			if (!sameKey(code, (int) cursor) && cursor != 0) {
				ret.action = SentenceModification.ACTION_APPEND;
			} else {
				ret.action = SentenceModification.ACTION_REPLACE;
			}
			ret.string = String.valueOf((char) code);
		} else {
			ret.action = SentenceModification.ACTION_NONE;
		}
		return ret;
	}

	public static SentenceModification parseChordingLetter(KeyElement key) {
		SentenceModification ret = new SentenceModification();
		ret.keyElement = new KeyElement(key);
		ret.string = "";

		int code = key.getNumber();
		if (code == TwidorConstants.KEY_BACKSPACE || code == TwidorConstants.KEY_DELETE) {
			ret.action = SentenceModification.ACTION_BACKSPACE;
		} else if (code != TwidorConstants.KEY_ENTER && code != TwidorConstants.KEY_EOL && code > 0) {
			ret.action = SentenceModification.ACTION_APPEND_ADVANCE;
			ret.string = String.valueOf((char) code);
		} else {
			ret.action = SentenceModification.ACTION_NONE;
		}
		return ret;

	}


	/**
	 * Returns a Vector containing the keystrokes that
	 * resulted in the entered text.  This can be used
	 * to calculate the typing speed and accuracy.
	 */
	static int timesCalled = 0;
	public Vector getEnteredKeystrokes() {
		//System.out.println("getEnteredKeyStrokes() called " + ++timesCalled + " times");
		Vector ret = new Vector();

		if (inputType == TwidorConstants.INPUT_MULTITAP) {
			int cursor = 0;
			for (int i = 0; i < getKeyPresses().size(); i++) {
				KeyElement key = (KeyElement) getKeyPresses().elementAt(i);
				int code = key.getNumber();
				if (code == -255) continue;
				//System.out.println("Sentence: " + sentence + " " + "i: " + i + " code: " + code + " ");
				
				int curCode = -1;
				if (0 <= cursor && cursor < ret.size()) {
					curCode = ((KeyElement) ret.elementAt(cursor)).getNumber();
				}
				
				if (code == TwidorConstants.KEY_MULTITAP ||
					code == TwidorConstants.KEY_RIGHT) {
					if (ret.size() <= cursor) {
						//	Nothing typed, so insert a space
						ret.addElement(new KeyElement(TwidorConstants.KEY_SPACE, key.getTime()));
					}
					
					cursor++;
				} else if (code == TwidorConstants.KEY_LEFT) {
					if (cursor > 0) {
						cursor--;
						ret.removeElementAt(ret.size() - 1);
					}
				} else if (code == TwidorConstants.KEY_SPACE) {
					ret.addElement(key);
					cursor++;
				} else if (code > 0 && code != TwidorConstants.KEY_MENTER) {
					if (!sameKey(code, curCode) && curCode != -1) {
						cursor++;
					}
					if (cursor < ret.size()) {
						ret.setElementAt(key, cursor);
					} else {
						ret.addElement(key);
					}
				}
			}
			
			if (ret.size() > 0) {
				KeyElement lastKey = (KeyElement) ret.lastElement();
				if (lastKey.getNumber() == TwidorConstants.KEY_MENTER) {
					ret.removeElementAt(ret.size() - 1);
				}
			}
			
		} else {
			for (int i = 0; i < getKeyPresses().size(); i++) {
				KeyElement key = (KeyElement)getKeyPresses().elementAt(i);
				int code = key.getNumber();
				if (code == TwidorConstants.KEY_BACKSPACE ||
						code == TwidorConstants.KEY_DELETE) {
					if (ret.size() > 0) ret.removeElementAt(ret.size() - 1);
				} else if (code != TwidorConstants.KEY_ENTER &&
						code != TwidorConstants.KEY_EOL &&
						code > 0) {
					ret.addElement(key);
				}
			}
		}
		return ret;
	}// end getEnteredKeystrokes()

	/**
	 * Returns an array of ints which are the key codes
	 * of each key that was pressed.
	 */
	public int [] getKeystrokes() {
		ArrayList strokes = new ArrayList();
		for (int i = 0; i < getKeyPresses().size(); i++) {
			KeyElement key = (KeyElement)getKeyPresses().elementAt(i);
			if (key.getNumber() > 0) {
				strokes.add(new Integer(key.getNumber()));
			}
		}

		return getIntArray(strokes);
	}// end getKeystrokes()

	/**
	 * Returns an array of ints that represent the transcribed text.
	 */
	public int [] getEnteredText() {
		Vector keystrokes = getEnteredKeystrokes();;
		if (keystrokes == null) return null;

		int [] enteredText = new int[keystrokes.size()];
		
		for (int i = 0; i < keystrokes.size(); i++) {
			enteredText[i] = ((KeyElement) keystrokes.elementAt(i)).getNumber();
		}
		
		return enteredText;
	}// end getEnteredText()

	public String getEnteredString() {
		int [] keystrokes = getEnteredText();
		String ret = "";
		
		for (int i = 0; i < keystrokes.length; i++) {
			ret += (char) keystrokes[i];
		}
		return ret;
	}
	
	/**
	 * Finds the minimum string distance between two strings,
	 * represented as arrays of integers.  Returns an array of
	 * two integers.  The first is the string distance, the
	 * second is the number of characters that were in A but
	 * not in B;
	 */
	public int [] distanceStats(int [] A, int [] B) {
		int [][] D = new int[A.length + 1][B.length + 1];
		int [][] missingFromA = new int[A.length + 1][B.length + 1];

		for (int i = 0; i <= A.length; i++) {
			D[i][0] = i;
			missingFromA[i][0] = 0;

		}

		for (int j = 0; j <= B.length; j++) {
			D[0][j] = j;
			missingFromA[0][j] = j;
		}

		for (int i = 1; i <= A.length; i++) {
			for (int j = 1; j <= B.length; j++) {
				int min;
				int notInA;
				int notInB;
				int substitution;

				notInB = D[i - 1][j] + 1;
				notInA = D[i][j - 1] + 1;
				substitution = D[i - 1][j - 1] + (A[i - 1] == B[j - 1] ? 0 : 1);
				

				
				min = notInB;
				missingFromA[i][j] = missingFromA[i - 1][j];

				if (substitution < min) {
					min = substitution;
					missingFromA[i][j] = missingFromA[i - 1][j - 1];
				}
				if (notInA < min) {
					min = notInA;
					missingFromA[i][j] = missingFromA[i][j - 1] + 1;
				}

				D[i][j] = min;
			}
		}// end distanceStats()

		int [] ret = new int[2];
		ret[0] = D[A.length][B.length];
		ret[1] = missingFromA[A.length][B.length];

		return ret;
	}// end distanceStats(int [], int [])

	/**
	 * Gets the minimum string distance between two strings.
	 */
	public int distance(int [] A, int [] B) {
		return distanceStats(A, B)[0];
	}// end distance(int [], int [])

	/**
	 * Returns the number of correct keystrokes entered
	 */
	public int getC() {
		int [] enteredText = getEnteredText();
		int [] sentence = getIntArray(this.sentence);

		if (enteredText == null) {
			//	If no text was entered, there was no correct
			//	text entered.
			return 0;
		}
		
		int [] stats = distanceStats(sentence, enteredText);
		int distance = stats[0];
		int extraTextChars = stats[1];

		//	The string distance includes characters that were
		//	entered but that were not part of the sentence.
		//	Entering extra characters should not decrease the
		//	number of correct characters reported, so we subtract
		//	it from the distance.
		return sentence.length - (distance - extraTextChars);
	}// end getC()

	/**
	 * Return the number of keystrokes which were incorrect,
	 * and were not fixed
	 */
	public int getINF() {
		int [] enteredText = getEnteredText();
		int [] sentence = getIntArray(this.sentence);

		if (enteredText == null) {
			//	In the case that no text was entered, all of
			//	the characters in the sentence were missing,
			//	so the length of the sentence is the string
			//	distance.
			return sentence.length;
		}

		return distance(enteredText, sentence);
	}// end getINF()

	/**
	 * Return the number of keystrokes in the fix class -
	 * ie those that are used for correcting mistakes.
	 */
	public int getF() {
		int retVal = 0;
		int [] keystrokes = getKeystrokes();

		if (keystrokes == null) return 0;

		for(int i = 0; i < keystrokes.length; i++) {
			if (keystrokes[i] == TwidorConstants.KEY_BACKSPACE ||
					keystrokes[i] == TwidorConstants.KEY_DELETE ||
					keystrokes[i] == TwidorConstants.KEY_LEFT) {
				retVal++;
			}
		}

		return retVal;
	}// end getF()

	/**
	 * Return the number of keystrokes which represent
	 * errors that were fixed.
	 */
	public int getIF() {
		//	Since the only editing keys are backspace and delete,
		//	and they do the same thing, we can take a shortcut here.
		//	They each delete one character, so F = IF;

		//	If the user presses backspace before entering any text,
		//	no characters are removed so that will cause some error
		//	in this function.
		return getF();
	}// end getIF()

	/**
	 * Returns the total error rate.  This includes both fixed
	 * and unfixed errors, and is a percentage of total text
	 * entered.
	 */
	public double getErrorRate() {
		double INF = getINF();
		double IF = getIF();
		double C = getC();
		
		return (INF + IF) / (C + INF + IF);
	}// end getErrorRate()

	/**
	 * Returns the accuracy percentage.
	 */
	public double getAccuracy() {
		return 1.0 - getErrorRate();
	}// end getAccuracy()

	/**
	 * Accessor to find out the CPS for this sentence
	 * @return double the CPS for this sentence
	 */
	public double getCPS () {
		long start = 0;
		long finish = 0;
		
		Vector keystrokes = getEnteredKeystrokes();

		int numCharsEntered = keystrokes.size() - 1;

		if (numCharsEntered > 0) {
			start = ((KeyElement)keystrokes.firstElement()).getTime();
			finish = ((KeyElement)keystrokes.lastElement()).getTime();
			long time = finish - start;
			double ret = ((double)numCharsEntered) / (time / 1000.0);
			if (TwidorConstants.bDEBUG) System.out.println("getCPS: time=" + time +
						", numCharsEntered=" + numCharsEntered + ", returning " + ret);
			return ret;
		}
		return 0;
	}// end getCPS ()

	public double getWPM() {
		return getCPS() * 12;
	}

	public int getLengthOfEnteredText() {
		return getEnteredText().length;
	}

	public long getTimeMillis() {
		Vector keystrokes = getEnteredKeystrokes();

		if (keystrokes.size() > 0) {
			long start = ((KeyElement) keystrokes.firstElement()).getTime();
			long finish = ((KeyElement) keystrokes.lastElement()).getTime();
			return finish - start;
		}
		return 0;
	}
	

	/**
	 * Accessor to fetch the error rate for this sentence
	 * @return double the error rate of this sentence
	 */
	public double getERR () {
		return getErrorRate();
	}// end getERR ()

	/**
	 * Prints out various statistics.  Useful for debugging.
	 */
	public void showStats() {
		int [] enteredText = getEnteredText();
		if (enteredText == null) return;

		String stats = "";

		stats += "Sentence:     " + sentence + "\n";
		stats += "Text entered: " + intArrayToString(enteredText) + "\n";
		stats += "Accuracy: " + getAccuracy() + "\n";
		stats += "C:        " + getC() + "\n";
		stats += "INF:      " + getINF() + "\n";
		stats += "IF:       " + getIF() + "\n";

		System.out.print(stats);
	}// end showStats()

	public String toString() {
		String ret = "";
		DecimalFormat fmt = new DecimalFormat("###.00");
		ret += "WPM: " + fmt.format(getWPM()) + "\t";
		ret += "Acc: " + fmt.format(getAccuracy() * 100) + "\t";
		ret += "C: " + getC() + "\t";
		ret += "INF: " + getINF() + "\t";
		ret += "F: " + getF() + "\t";
		return ret;
	}


	/**
	 * Function for writing all stats to a file.
	 * @param String the name of the file to output to
	 * @return int the exit code of this function;
	 * (-1 for error, 0 for no data, >0 for lines written)
	 */
	public int writeStats (String dest) {
		FileWriter fwFile;
		BufferedWriter bwBuffer;
		KeyElement temp;
		int count = 0;
		if (getKeyPresses().size() <= 0 || getSentence() == null) {
			/* No need to bother saving */
			return count;
		}
		try {
			fwFile = new FileWriter(dest);
			bwBuffer = new BufferedWriter(fwFile);
			
			bwBuffer.write(getSentence());
			bwBuffer.newLine();

			if (inputType == TwidorConstants.INPUT_KEYBOARD) {
				bwBuffer.write("Keyboard");
			} else if (inputType == TwidorConstants.INPUT_MULTITAP) {
				bwBuffer.write("Multi-tap");
			} else {
				bwBuffer.write("Chording");
			}

			if (TwidorConstants.currentState == TwidorConstants.STATE_WARMUP) {
				bwBuffer.write(" warmup");
			}

			bwBuffer.newLine();

			for (int i = 0; i < getKeyPresses().size(); i++) {
				temp = (KeyElement)getKeyPresses().elementAt(i);
				//bwBuffer.write(temp.getTime() + "\t" + temp.toString());
				bwBuffer.write(temp.toString());
				bwBuffer.newLine();
				count++;
			}

			String stats = "";

			stats += "Accuracy: " + getAccuracy() + "\n";
			stats += "C:        " + getC() + "\n";
			stats += "INF:      " + getINF() + "\n";
			stats += "IF:       " + getIF() + "\n";

			//System.out.print(stats);

			//bwBuffer.write(stats);

			bwBuffer.close();
			fwFile.close();
		}
		catch (Exception e) {
			e.printStackTrace();
			return -1;
		}
		return count;
	}// end writeStats (String)

	public static Stats readStats(String source) {
		FileReader frFile;
		BufferedReader brBuffer;
		KeyElement temp;
		int count = 0;
		Stats ret = new Stats();
		ret.setKeyPresses(new Vector());
		
		try {
			frFile = new FileReader(source);
			brBuffer = new BufferedReader(frFile);

			ret.sentence = brBuffer.readLine();
			String type = brBuffer.readLine();

			if (type == null) return null;

			if (type.startsWith("Keyboard"))
				ret.inputType = TwidorConstants.INPUT_KEYBOARD;
			else if (type.startsWith("Multi-tap"))
				ret.inputType = TwidorConstants.INPUT_MULTITAP;
			else if (type.startsWith("Chording"))
				ret.inputType = TwidorConstants.INPUT_TWIDDLER;

			String line;
			while ((line = brBuffer.readLine()) != null) {
				temp = KeyElement.fromString(line);
				ret.getKeyPresses().addElement(temp);
			}

			brBuffer.close();
			frFile.close();
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		
		return ret;
	}

	/**
	 * Gets aggregate statistics for a vector of Stats objects.
	 * Returns an array of doubles which includes the total
	 * characters per second, words per minute, and accuracy
	 * percentage.
	 */
	public static double [] getAggregateStats(Vector statVector) {
		double iCPS = 0, CPS = 0, iERR = 0, ERR = 0;
		double iCHARS = 0, CHARS = 0;
		long iTIME = 0, TIME = 0;
		double C = 0, INF = 0, IF = 0;

		for (int i = 0; i < statVector.size(); i++) {
			Stats stats = (Stats) statVector.elementAt(i);
			C += stats.getC();
			INF += stats.getINF();
			IF += stats.getIF();

			iCHARS = stats.getLengthOfEnteredText() - 1;
			CHARS += iCHARS;

			iTIME = stats.getTimeMillis();
			TIME += iTIME;
			
			//	Taking the average of the error rates for each
			//	sentence isn't quite correct because the
			//	sentences are not all the same length.			
			iERR = stats.getERR();
			//ERR += iERR;
		}
		//	Taking the average of the error rates for each
		//	sentence isn't quite correct because the
		//	sentences are not all the same length.
		//	So compute the correct error here.
		ERR = (INF + IF) / (C + INF + IF);

		double [] ret = new double[3];

		double accuracy = 1.0 - ERR;
		accuracy = (double) ((int) (accuracy * 1000)) / 10.0;

		ret[0] = (double) ((int)(CHARS / (TIME / 1000.0) * 100))/100.0;		//	Chars per second
		ret[1] = (double)((int)(CHARS / (TIME / 1000.0) * 12 * 100))/100.0;	//	WPM
		ret[2] = accuracy;

		return ret;
	}// end getAggregateStats(Vector)

}// end class Stats
