/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.util.*;
import java.util.zip.*;
import java.text.DecimalFormat;
import java.io.*;
import java.text.DateFormat;
//import org.jfree.data.Statistics;

/**
 * Summary description for StatsBrowser.
 */
public class StatsBrowser
{
	public String username;
	public int session = -1;
	public int block = -1;
	public int sentence = -1;

	public static void doKey(KeyElement key, int num) {
		if (num == -2) {
			key.setButton(1, true);
			key.setButton(4, true);
		} else if (num != -1) {
			key.setButton(num, true);
		}
	}

	public Double [] toDoubleArray(ArrayList list) {
		Double [] ret = new Double[list.size()];
		for (int i=0; i<list.size(); i++) {
			ret[i] = (Double) list.get(i);
		}
		return ret;
	}

	public void browse() {
		ArrayList statsFilters = new ArrayList();
		FileGenerator generator = new FileGenerator();

		try {
			BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
			String input = "";
			while (!input.equalsIgnoreCase("exit")) {
				input = stdin.readLine();
				DecimalFormat fmt = new DecimalFormat("####.####");

				String [] words = input.split(" ");

				if (words[0].equalsIgnoreCase("predictcharspeeds")) {
					BufferedWriter statsWriter = new BufferedWriter(new FileWriter("charstats.txt"));
					String out = "";

					KeyMap keymap = new KeyMap();
					keymap.readKeyMap("keymap.txt");
					TypingModel typingModel = new TypingModel(keymap);

					for (char ch1 = 'a'; ch1 != (' ' +  1); ch1 = (ch1 == 'z' ? ' ' : (char) (ch1 + 1))) {
						for (char ch2 = 'a'; ch2 != (' ' + 1); ch2 = (ch2 == 'z' ? ' ' : (char) (ch2 + 1))) {

							double delay = typingModel.getDelay(ch1, ch2);
							out += fmt.format(delay) + "\t";
						}
						out += "\n";
					}
					statsWriter.write(out);
					System.out.print(out);
					statsWriter.close();


				} else if (words[0].equalsIgnoreCase("charspeeds")) {
					BufferedWriter statsWriter = new BufferedWriter(new FileWriter("charstats.txt"));
					String out;

					KeyMap keymap = new KeyMap();
					keymap.readKeyMap("keymap.txt");

					statsFilters.clear();
					statsFilters.add(StatsFilter.getLastTwoSessionsFilter());
					statsFilters.add(StatsFilter.getChordingFilter());
					statsFilters.add(StatsFilter.getAccuracyFilter(1.0));

					String [] users = getUsers();
					for (int user=0; user<users.length; user++) {
						ArrayList results = generator.getStats(users[user], statsFilters);

						out = "User: " + users[user] + "\n";

						Map charTimings = new HashMap();
						Iterator it = results.iterator();
						while (it.hasNext()) {
							Stats stats = (Stats) it.next();
							stats.getTransitionDelays(charTimings);
							//System.out.println("Parsed sentence: " + stats.getSentence());
						}
						out += "\t";
						String nextLine = "\t";
						for (char ch = 'a'; ch <= 'z'; ch++) {
							out += ch + "\t\t\t";
							nextLine += "first press\ttravel time\tsecond press\t";
						}
						out += "SP\n";
						out += nextLine + "\n";
						for (char ch1 = 'a'; ch1 != (' ' +  1); ch1 = (ch1 == 'z' ? ' ' : (char) (ch1 + 1))) {
							KeyElement k1 = keymap.getKey(String.valueOf(ch1));
							out += "" + ch1 + "\t";
							//System.out.println("k1: " + k1);
							for (char ch2 = 'a'; ch2 != (' ' + 1); ch2 = (ch2 == 'z' ? ' ' : (char) (ch2 + 1))) {
								
								KeyElement k2 = keymap.getKey(String.valueOf(ch2));
								KeyElement.KeyPair keyPair = new KeyElement.KeyPair(k1, k2);
								ArrayList [] delays = (ArrayList []) charTimings.get(keyPair);

								if (delays == null) {
									out += "\t\t\t";
								} else {
									for (int i=0; i<3; i++) {
										Double [] array = new Double[delays[i].size()];
										for (int j=0; j<array.length; j++) {
											array[j] = new Double(((Long) delays[i].get(j)).doubleValue() / 1000.0);
										}
										double average = Statistics.getCleanedAverage(array);
										if (Double.isNaN(average)) {
											out += "\t";
										} else {
											out += fmt.format(average) + "\t";
										}
									}
									System.out.print(out);
									statsWriter.write(out);
									statsWriter.flush();
									out = "";
								}

							}
							out += "\n";
							System.out.print(out);
							statsWriter.write(out);
							statsWriter.flush();
							out = "";
						}
						out += "\n";
					}

					statsWriter.close();

				} else if (words[0].equalsIgnoreCase("sentencespeeds")) {
					BufferedReader sentenceReader = new BufferedReader(new FileReader("phrases.txt"));
					BufferedWriter statsWriter = new BufferedWriter(new FileWriter("sentencestats.txt"));
					String sentence;
					String out;

					KeyMap keymap = new KeyMap();
					keymap.readKeyMap("keymap.txt");

					TypingModel typingModel = new TypingModel(keymap);

					while ((sentence = sentenceReader.readLine()) != null) {
						statsFilters.clear();
						statsFilters.add(StatsFilter.getLastTwoSessionsFilter());
						statsFilters.add(StatsFilter.getSentenceFilter(sentence));
						statsFilters.add(StatsFilter.getChordingFilter());
						statsFilters.add(StatsFilter.getAccuracyFilter(1.0));

						System.out.print(sentence + "\t");
						statsWriter.write(sentence + "\t");

						out = fmt.format(typingModel.predictSentenceDelay(sentence)) + "\t";
						System.out.print(out);
						statsWriter.write(out);

						String [] users = getUsers();
						for (int i=0; i<users.length; i++) {
							ArrayList results = generator.getStats(users[i], statsFilters);
							Iterator it = results.iterator();
							while (it.hasNext()) {
								Stats stats = (Stats) it.next();
								double delay = stats.getTimeMillis() / 1000.0;
								out = fmt.format(delay) + "\t";
								System.out.print(out);
								statsWriter.write(out);
							}
						}
						System.out.println();
						statsWriter.write("\n");
					}
					sentenceReader.close();
					statsWriter.close();

				} else if (words[0].equalsIgnoreCase("test")) {
					BufferedReader tableReader = new BufferedReader(new FileReader("chordingtiming.txt"));
					BufferedWriter statsWriter = new BufferedWriter(new FileWriter("chordtimestats.txt"));
					String tableLine;
					while ((tableLine = tableReader.readLine()) != null) {
						String [] columns = tableLine.split("\t");

						statsFilters.clear();
						statsFilters.add(StatsFilter.getSentenceFilter(columns[0]));

						for (int direction = 0; direction < 2; direction++) {

							String fname = "data/timing/thad" + columns[0];
							if (direction == 1) fname += " reversed";
							fname += ".txt";
							BufferedWriter allWriter = new BufferedWriter(new FileWriter(fname));

							KeyElement k1 = new KeyElement();
							KeyElement k2 = new KeyElement();
							try {
								doKey((direction == 0) ? k1 : k2, Integer.parseInt(columns[1]));
								doKey((direction == 0) ? k1 : k2, Integer.parseInt(columns[2]));
								doKey((direction == 0) ? k2 : k1, Integer.parseInt(columns[3]));
								doKey((direction == 0) ? k2 : k1, Integer.parseInt(columns[4]));

								//if (k1.equals(k2)) continue;

								if (k1.equals(k2) && direction == 1) {
									continue;
								}
							} catch (NumberFormatException e) {
								System.out.println("Error parsing line: " + tableLine);
								continue;
							}

							String [] users = getUsers();
							ArrayList charTimes = new ArrayList();
							ArrayList [] userDelays = new ArrayList[3];
							ArrayList [] userStdDevs = new ArrayList[3];
							ArrayList [] numOutliers = new ArrayList[3];
							for (int i=0; i<3; i++) {
								userDelays[i] = new ArrayList();
								userStdDevs[i] = new ArrayList();
								numOutliers[i] = new ArrayList();
							}
							for (int i=0; i<users.length; i++) {

								ArrayList results = generator.getStats(users[i], statsFilters);
								Iterator it = results.iterator();

								while (it.hasNext()) {
									Stats stats = (Stats) it.next();
									ArrayList [] data = stats.getTransitionDelays(k1, k2);
									Long [][] data2 = new Long [3][data[0].size()];
									for (int k=0; k<3; k++) {
										allWriter.write(users[i]);
										if (k==0) allWriter.write(" first held");
										else if (k==1) allWriter.write(" between keys");
										else allWriter.write(" second held");

										for (int j=0; j<data[0].size(); j++) {
											Long delay = (Long) data[k].get(j);
											data2[k][j] = delay;

											allWriter.write("\t" + String.valueOf(delay));

											//if (users[i].equals("125")) System.out.println("Delay: " + delay);
										}
										allWriter.write("\n");

										double delayAvg = Statistics.getAverage(data2[k]);
										double stdDev = Statistics.getStdDev(data2[k]);

										if (Double.isNaN(stdDev)) {
											System.out.println("NaN- user: " + users[i] + "\tk: " + k);
										} else if (!Double.isNaN(delayAvg)) {
											userDelays[k].add(new Double(delayAvg));
											userStdDevs[k].add(new Double(stdDev));
											
											int outliers = 0;
											for (int j=0; j<data[k].size(); j++) {
												double diff = Math.abs(delayAvg - ((Long) data[k].get(j)).longValue());
												if (diff > stdDev * 2) {
													outliers++;
													//System.out.print(fmt.format(diff / stdDev) + "\t");
												}
											}
											numOutliers[k].add(new Integer(outliers));
											//if (outliers > 0) System.out.println("\t\ttotal: " + data[k].size());
										}
									}

									//userDelays[i] = new Double(Statistics.getAverage(data2));

									//System.out.println("user " + users[i] + ": " + Statistics.getAverage(data2) + "\tstddev:" + Statistics.getStdDev(data2));
								}
							}
							//System.out.println();

							allWriter.close();

							for (int i=0; i<3; i++) {
								Double [] array = toDoubleArray(userDelays[i]);
								double avg = Statistics.getAverage(array);
								String line = fmt.format(avg) + "\t";
								double stdev = Statistics.getStdDev(array);
								//line += fmt.format(stdev) + "\t";
								avg = Statistics.getAverage(toDoubleArray(userStdDevs[i]));
								line += fmt.format(avg) + "\t";
								/*
								 if (Double.isNaN(avg)) {
								 for (int j = 0; j < userDelays[i].length; j++) {
								 System.out.print("" + userDelays[i][j] + "\t");
								 }
								 System.out.println();
								 }
								 */
								//System.out.print(Statistics.getAverage(userDelays[i]));

								statsWriter.write(line);
								//System.out.print(line);
							}

							//if (direction == 1 || k1.equals(k2)) {
								String endLine = columns[0];
								if (direction == 1) endLine += " other direction";
								endLine += "\n";

								statsWriter.write(endLine);
								System.out.print(endLine);
							//}

						}
						//statsWriter.write(columns[0] + "\n");
						//System.out.println(columns[0]);
						

						//System.out.println("Average: " + Statistics.getAverage(userDelays));
						//System.out.println("Stdev:   " + Statistics.getStdDev(userDelays));
					}
					statsWriter.close();
					tableReader.close();
				} else if (words[0].equalsIgnoreCase("predict")) {
					LanguageModel languageModel = new LanguageModel();
					languageModel.read("digramfrequencies.txt");

					KeyMap keymap = new KeyMap();
					keymap.readKeyMap("keymap.txt");

					TypingModel typingModel = new TypingModel(keymap);

					double wpm = languageModel.predictWPM(typingModel);

					System.out.println("predicted WPM: " + wpm);

					

				} else if (words[0].equalsIgnoreCase("user")) {
					if (words.length < 2) {
						System.out.println("Please specify username.");
					} else {
						String backup = username;
						username = words[1];
						if (userPath() == null) {
							username = backup;
							System.out.println("Bad user name");
						}
						//UserData user = UserData.read(username);

					}
				} else if (words[0].equalsIgnoreCase("session")) {
					if (words.length < 2) {
						System.out.println("Please specify a session number.");
					} else {
						try {
							int backup = session;
							session = Integer.parseInt(words[1]);
							if (sessionPath() == null) {
								session = backup;
								System.out.println("Session does not exist.");
							}
						} catch (NumberFormatException e) {
							System.out.println("Unable to parse session number.");
						}
					}
				} else if (words[0].equalsIgnoreCase("block")) {
					if (words.length < 2) {
						System.out.println("Please specify a block number.");
					} else {
						try {
							int backup = block;
							block = Integer.parseInt(words[1]);
							if (blockPath() == null) {
								block = backup;
								System.out.println("Block does not exist.");
							}
						} catch (NumberFormatException e) {
							System.out.println("Unable to parse block number.");
						}
					}
				} else if (words[0].equalsIgnoreCase("sentence")) {
					if (words.length < 2) {
						System.out.println("Please specify a sentence number.");
					} else {
						try {
							int backup = sentence;
							sentence = Integer.parseInt(words[1]);
							if (sentenceFile() == null) {
								sentence = backup;
								System.out.println("Sentence does not exist.");
							}
						} catch (NumberFormatException e) {
							System.out.println("Unable to parse sentence number.");
						}
					}
				} else if (words[0].equalsIgnoreCase("pwd")) {
					if (sentenceFile() != null) System.out.println(sentenceFile());
					else if (blockPath() != null) System.out.println(blockPath());
					else if (sessionPath() != null) System.out.println(sessionPath());
					else if (userPath() != null) System.out.println(userPath());
					else System.out.println(TwidorConstants.PATH_TO_DATA);
				} else if (words[0].equalsIgnoreCase("stats")) {
					String filename = sentenceFile();
					if (filename == null) {
						System.out.println("You must first select a valid sentence.");
					} else {
						Stats stats = Stats.readStats(filename);
						if (stats == null) {
							System.out.println("Unable to read stats.");
						} else {
							System.out.println(stats.toString());
							System.out.println("Transcribed text: " + stats.getEnteredString());
						}
						System.out.println("user: " + getUserFromFilename(filename));
						System.out.println("session: " + getSessionNumber(filename));
						System.out.println("block: " + getBlockNumber(filename));
						System.out.println("sentence: " + getSentenceNumber(filename));
					}
				} else if (words[0].equalsIgnoreCase("blockstats")) {
					System.out.println(blockStats(true, ""));
				} else if (words[0].equalsIgnoreCase("sessionstats")) {
					System.out.println(sessionStats(true, ""));
				}

			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
    
	public String userPath() {
		return userPath(username);
	}
	public String userPath(String username) {
		if (username == null) return null;
		String path = TwidorConstants.PATH_TO_DATA + username;
		File file = new File(path);
		if (file.exists() && file.isDirectory()) return path;
		else return null;
	}
	public String sessionPath() {
		String path = userPath();
		if (path == null || session == -1) return null;
		path += File.separator + "Session" + TwidorStudy.formatint(session);
		File file = new File(path);
		if (file.exists() && file.isDirectory()) return path;
		else return null;
	}
	public String blockPath() {
		return blockPath(block);
	}
	public String blockPath(int block) {
		String path = sessionPath();
		if (path == null || block == -1) return null;
		path += File.separator + "Block" + TwidorStudy.formatint(block);
		File file = new File(path);
		if (file.exists() && file.isDirectory()) return path;
		else return null;
	}
	public String sentenceFile() {
		return sentenceFile(sentence);
	}
	public String sentenceFile(int sentence) {
		String path = blockPath();
		if (path == null || sentence == -1) return null;
		path += File.separator + "Sentence" + TwidorStudy.formatint(sentence);
		File file = new File(path);
		if (file.exists() && file.isFile()) return path;
		else return null;
	}

	public static int getSentenceNumber(String filename) {
		try {
			File file = new File(filename);
			String filestring = file.getName();
			String numString = filestring.substring("Sentence".length());
			return Integer.parseInt(numString);
		} catch (Exception e) {
			System.out.println("Unable to parse filename for sentence number: " + filename);
			return -1;
		}
	}

	public static int getBlockNumber(String filename) {
		try {
			File file = new File(filename);
			String filestring = file.getParentFile().getName();
			String numString = filestring.substring("Block".length());
			return Integer.parseInt(numString);
		} catch (Exception e) {
			System.out.println("Unable to parse filename for block number: " + filename);
			return -1;
		}
	}

	public static int getSessionNumber(String filename) {
		try {
			File file = new File(filename);
			String filestring = file.getParentFile().getParentFile().getName();
			String numString = filestring.substring("Session".length());
			return Integer.parseInt(numString);
		} catch (Exception e) {
			System.out.println("Unable to parse filename for Session number: " + filename);
			return -1;
		}
	}

	public static String getUserFromFilename(String filename) {
		File file = new File(filename);
		return file.getParentFile().getParentFile().getParentFile().getName();
	}

	public String [] getUsers() {
		return new String[] {"125", "127", "128", "129", "130", "132", "133", "134", "135", "137"};
		//return new String[] {"666"};
	}

	public String blockStats(boolean showSentenceStats, String prefix) {
		String path = blockPath();
		if (path == null) return null;

		String ret = "";
		Vector sentences = new Vector();
		for (int s = 1;;s++) {
			String file = sentenceFile(s);
			if (file == null) break;
			else {
				Stats stats = Stats.readStats(file);
				if (stats == null) {
					System.out.println("Error reading stats: " + file);
				} else {
					sentences.addElement(stats);
					if (showSentenceStats) {
						ret += prefix + "   Sentence " + s + ":\t" + stats.toString() + "\n";
					}
				}
			}
		}

		double [] stats = Stats.getAggregateStats(sentences);

		ret = "WPM: " + stats[1] + "\tAcc: " + stats[2] + "\n" + ret;


		return ret;
	}
	
	public Block readBlock(String path) {
		Block block = new Block();
		for (int s = 1;; s++) {
			String file = path + File.separator + "Sentence" + TwidorStudy.formatint(s);
			if (!new File(file).exists()) break;
			//System.out.println(file);
			Stats stats = Stats.readStats(file);
			if (stats != null) block.addSentence(stats);
			else System.out.println("Unable to read: " + file);
		}
		return block;
	}
	
	public String sessionStats(boolean showBlockStats, String prefix) {
		String path = sessionPath();
		if (path == null) return null;
		
		String ret = "";
		Vector blocks = new Vector();
		Vector chording = new Vector();
		Vector multitap = new Vector();
		Vector allSentences = new Vector();
		
		for (int b = 1;;b++) {
			path = blockPath(b);
			if (path == null) break;
			//System.out.println("Block " + b);
			else {
				Block block = readBlock(path);
				if (block.size() > 0) {
					Stats stats = block.getSentence(0);
					blocks.addElement(block);
					allSentences.addAll(block.getSentences());
					if (stats.getInputType() == TwidorConstants.INPUT_TWIDDLER)
						chording.addAll(block.getSentences());
					else if (stats.getInputType() == TwidorConstants.INPUT_MULTITAP)
						multitap.addAll(block.getSentences());
				}
			}
		}
		
		if (blocks.size() == 0) return "No blocks completed.";
		
		long start = ((KeyElement) ((Block) blocks.firstElement()).getSentence(0).getEnteredKeystrokes().firstElement()).getTime();
		Block lastBlock = (Block) blocks.lastElement();
		long finish = ((KeyElement) (lastBlock).getSentence(lastBlock.size() - 1).getEnteredKeystrokes().lastElement()).getTime();
		
		double [] chordingStats = Stats.getAggregateStats(chording);
		double [] multitapStats = Stats.getAggregateStats(multitap);
		double [] allStats = Stats.getAggregateStats(allSentences);
		
		DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
		
		ret += "Started: " + df.format(new Date(start)) + "\n";
		ret += "Ended: " + df.format(new Date(finish)) + "\n";
		
		ret += "Chording WPM:  " + chordingStats[1] + "\tACC: " + chordingStats[2] + "\tAdj WPM: " + chordingStats[1] * chordingStats[2] / 100.0 + "\n";
		ret += "Multi-tap WPM: " + multitapStats[1] + "\tACC: " + multitapStats[2] + "\tAdj WPM: " + multitapStats[1] * multitapStats[2] / 100.0 + "\n";
		ret += "Aggregate WPM: " + allStats[1] + "\tACC: " + allStats[2] + "\tAdj WPM: " + allStats[1] * allStats[2] / 100.0 + "\n";
		
		

		return ret;
	}

	public static void main(String [] args) {
		new StatsBrowser().browse();
	}
}
