/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;

/**
 * Summary description for StatsFilter.
 */
public class StatsFilter
{
	protected boolean fullBlock = false;

	//	Returns Stats object if it passes the filter.  If stats passed in
	//	is null, it will be read from the file if necessary.
	public Stats filter(String filename, Stats stats) {
		if (stats == null) stats = Stats.readStats(filename);
		return stats;
	}
	public boolean appliesToBlock() {
		return fullBlock;
	}

	public static StatsFilter getChordingFilter() {
		StatsFilter ret = new StatsFilter() {
			public Stats filter(String filename, Stats stats) {
				if (stats == null) stats = Stats.readStats(filename);
				if (stats.getInputType() == TwidorConstants.INPUT_TWIDDLER &&
					 !stats.getSentence().equals("abcd efgh ijkl") &&
					 !stats.getSentence().equals("mnop qrst uvwx yz")) return stats;
				else return null;
			}
		};
		ret.fullBlock = true;

		return ret;
	}

	public static StatsFilter getSentenceFilter(final String sentence) {
		StatsFilter ret = new StatsFilter() {
			String matchSentence = sentence;
			public Stats filter(String filename, Stats stats) {
				if (stats == null) {
					try {
						BufferedReader in = new BufferedReader(new FileReader(filename));
						String s = in.readLine();
						in.close();
						if (s == null) return null;
						if (s.equals(matchSentence)) return Stats.readStats(filename);
						return null;
					} catch (IOException e) {
						e.printStackTrace();
						return null;
					}
				} else {
					if (stats.getSentence().equals(matchSentence)) return stats;
					else return null;
				}
			}
		};

		return ret;
	}

	public static StatsFilter getAccuracyFilter(final double accuracy) {
		StatsFilter ret = new StatsFilter() {
			double matchAccuracy = accuracy;
			public Stats filter(String filename, Stats stats) {
				if (stats == null) stats = Stats.readStats(filename);
				if (stats.getAccuracy() >= matchAccuracy) return stats;
				else return null;
			}
		};
		return ret;
	}

	public static StatsFilter getLastTwoSessionsFilter() {
		StatsFilter ret = new StatsFilter() {
			public Stats filter(String filename, Stats stats) {
				String user = StatsBrowser.getUserFromFilename(filename);
				int session = StatsBrowser.getSessionNumber(filename);

				if ((user.equals("125") && session >= 21) ||
					(user.equals("127") && session >= 24) ||
					(user.equals("128") && session >= 23) ||
					(user.equals("129") && session >= 23) ||
					(user.equals("130") && session >= 20) ||
					(user.equals("132") && session >= 19) ||
					(user.equals("133") && session >= 19) ||
					(user.equals("134") && session >= 21) ||
					(user.equals("135") && session >= 21) ||
					(user.equals("137") && session >= 23)) {
					
					if (stats == null) stats = Stats.readStats(filename);
					return stats;
				}


				return null;
			}
		};
		ret.fullBlock = true;

		return ret;
	}
}
