/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.util.Calendar;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * StatsPanel.java, the class that defines how stats are rendered to the
 * screen. Stats such as typing speed, corrected error rate, uncorrected error
 * rate, and all those other ones I'm going to have to re-read papers to
 * remember.
 *
 * Revisions:
 * 		0.5	17 July 2003
 * 			Completed Tutor
 * 		0.1	10 June 2003
 * 			Created class StatsPanel
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 17 July 2003
 */
public class StatsPanel extends JPanel {

	/**
	 * internal variables
	 */
	private Vector myStats;
	private JLabel myCPS; /* Characters per second */
	private JLabel myWPM; /* Theoretical words per minute */
	private JLabel myERR; /* Error rate */
	private JLabel myICPS; /* instantaneous cps */
	private JLabel myIWPM; /* instantaneous wpm */
	private JLabel myIERR; /* instantaneous err */
	private boolean saveStats;
	private boolean acceptInput = true;
	private TwidorConstants myDefaults;

	/**
	 * default constructor
	 */
	public StatsPanel (TwidorConstants twidorDefaults) {
		setDefaults(twidorDefaults);
		myStats = new Vector();
		setBackground(getDefaults().TEXT_BACKGROUND);
		setVisible(true);
		if (getDefaults().STATS_SHOW) {
			showPanel();
		}
		setSave(getDefaults().LOG_ENABLE);
	}// end StatsPanel ()

	/**
	 * modifier for the defaults entry
	 * @param TwidorConstants the new defaults
	 */
	private void setDefaults (TwidorConstants twidorDefaults) {
		myDefaults = twidorDefaults;
	}// end setDefaults (TwidorConstants)

	/**
	 * accessor for the defaults entry
	 * @return TwidorConstants the defaults
	 */
	private TwidorConstants getDefaults () {
		return myDefaults;
	}// end getDefaults ()

	public void enableInput(boolean enable) {
		acceptInput = enable;
	}

	/**
	 * set up the internals of this panel
	 */
	public void showPanel () {
		setVisible(false);
		removeAll();
		setBackground(getDefaults().TEXT_BACKGROUND);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		JPanel center = new JPanel();
		center.setAlignmentY(Component.CENTER_ALIGNMENT);
		center.setAlignmentX(Component.CENTER_ALIGNMENT);
		center.setLayout(new GridLayout(3, 3));
		center.setBackground(getDefaults().TEXT_BACKGROUND);

		/**
		 * *blank*	avg	/	last
		 * CPS		myCPS	/	myICPS
		 * WPM		myWPM	/	myIWPM
		 * ACC		myERR	/	myIERR
		 */

		JLabel temp = quickLabel("");
		center.add(temp);
		temp = quickLabel("avg");
		temp.setPreferredSize(new Dimension(80,24));
		temp.setHorizontalAlignment(SwingConstants.TRAILING);
		center.add(temp);
		/*
		temp = quickLabel("/");
		temp.setPreferredSize(new Dimension(12,24));
		temp.setHorizontalAlignment(SwingConstants.CENTER);
		center.add(temp);
		*/
		temp = quickLabel("last");
		temp.setPreferredSize(new Dimension(80,24));
		temp.setHorizontalAlignment(SwingConstants.TRAILING);
		center.add(temp);

		
		//center.add(quickLabel("CPS"));
		myCPS = quickLabel("0.0");
		myCPS.setHorizontalAlignment(SwingConstants.TRAILING);
		//center.add(myCPS);
		
		
		/*
		temp = quickLabel("/");
		temp.setHorizontalAlignment(SwingConstants.CENTER);
		center.add(temp);
		*/
		
		
		myICPS = quickLabel("0.0");
		myICPS.setHorizontalAlignment(SwingConstants.TRAILING);
		//center.add(myICPS);
		

		center.add(quickLabel("WPM"));
		myWPM = quickLabel("0");
		myWPM.setHorizontalAlignment(SwingConstants.TRAILING);
		center.add(myWPM);
		/*
		temp = quickLabel("/");
		temp.setHorizontalAlignment(SwingConstants.CENTER);
		center.add(temp);
		*/
		myIWPM = quickLabel("0");
		myIWPM.setHorizontalAlignment(SwingConstants.TRAILING);
		center.add(myIWPM);

		center.add(quickLabel("ACC"));
		myERR = quickLabel("0.0");
		myERR.setHorizontalAlignment(SwingConstants.TRAILING);
		center.add(myERR);
		/*
		temp = quickLabel("/");
		temp.setHorizontalAlignment(SwingConstants.CENTER);
		center.add(temp);
		*/
		myIERR = quickLabel("0.0");
		myIERR.setHorizontalAlignment(SwingConstants.TRAILING);
		center.add(myIERR);

		center.setMaximumSize(center.getPreferredSize());

		add(Box.createVerticalGlue());
		add(center);
		add(Box.createVerticalGlue());
		setVisible(true);
	}// end initPanel ()

	/**
	 * remove the internals of the panel
	 */
	public void hidePanel () {
		setVisible(false);
		removeAll();
		setBackground(getDefaults().TEXT_BACKGROUND);
		setVisible(true);
	}// end hidePanel ()

	/**
	 * Quicky helper function for repetitive tasks
	 */
	private JLabel quickLabel (String label) {
		JLabel temp = new JLabel(label);
		temp.setFont(new Font("Monospaced", Font.PLAIN, 20));
		temp.setForeground(getDefaults().TEXT_DEFAULT);
		temp.setBackground(getDefaults().TEXT_BACKGROUND);
		temp.setHorizontalAlignment(SwingConstants.LEADING);
		return temp;
	}// end quickLabel ()

	/**
	 * Called when a new Sentence is being displayed
	 */
	public void nextSentence (String sentence) {
		getStats().add(new Stats(sentence, currentInputType()));
		if (isVisible()) {
			reLabel();
		}
	}// end nextSentence (String)

	/**
	 * Accessor for the Stats variable
	 * @return Stats the current Stat
	 */
	public Vector getStats () {
		return myStats;
	}// end getStats ()

	public Stats currentStats() {
		if (myStats.size() == 0) return null;
		return (Stats) myStats.lastElement();
	}

	/**
	 * Accessor for the Saving variable
	 * @return boolean whether or not we save
	 */
	public boolean getSave () {
		return saveStats;
	}// end getSave ()

	/**
	 * Modifier for the Saving variable
	 * @param boolean the new value
	 */
	public void setSave (boolean newvalue) {
		saveStats = newvalue;
	}// end saveStats (boolean)

	/**
	 * Informs the StatsPanel that a character was typed.
	 * StatsPanel takes the appropriate action and updates the labels.
	 * @param KeyElement the typed Key
	 */
	public void charTyped (KeyElement key) {
		if (!acceptInput) return;
		int current = getStats().size() - 1;
		((Stats)getStats().elementAt(current)).addPressed(key);
	}// end charTyped (KeyElement, KeyElement, long)

	/**
	 * Informs the StatsPanel that the non-keyboard interface got a
	 * buttonpress.
	 * @param KeyElement the new state
	 */
	public void charState (KeyElement key) {
		if (!acceptInput) return;
		int current = getStats().size() - 1;
		((Stats)getStats().elementAt(current)).addState(key);
	}// end charState (KeyElement)

	/**
	 * Accessor that re-draws what goes on the labels.
	 */
	public void reLabel () {
		reLabel(false);
	}

	public void reLabel (boolean includeLastSentence) {
		if (getDefaults().bDEBUG)
			System.out.println("StatsPanel: Relabeling.");

		//	Get aggregate stats
		Vector statsVector = (Vector) getStats().clone();

		if (statsVector.size() <= 1) return;

		if (!includeLastSentence)
			statsVector.removeElementAt(statsVector.size() - 1);

		double [] statValues = Stats.getAggregateStats(statsVector);
		double CPS = statValues[0];
		double WPM = statValues[1];
		double ACC = statValues[2];

		myCPS.setText(String.valueOf(((int)(CPS * 100)) / 100.0));
		myWPM.setText(String.valueOf(((int)(WPM * 100)) / 100.0));
		//myERR.setText("" + (((int) (ACC * 100)) / 100) + "%");
		//myCPS.setText("" + CPS);
		//myWPM.setText("" + WPM);
		myERR.setText("" + ACC + "%");



		//	Get stats for last sentence;
		Stats stats = (Stats) statsVector.elementAt(statsVector.size() - 1);

		double iCPS = stats.getCPS();
		double iWPM = iCPS * 12;
		double iACC = ((int) (stats.getAccuracy() * 1000)) / 10.0;

		myICPS.setText(String.valueOf(((int)(iCPS * 100)) / 100.0));
		myIWPM.setText(String.valueOf(((int)(iWPM * 100)) / 100.0));
		//myIERR.setText("" + (((int) (iACC * 100)) / 100) + "%");
		//myICPS.setText("" + iCPS);
		//myIWPM.setText("" + iWPM);
		myIERR.setText("" + iACC + "%");

	}// end reLabel ()

	/**
	 * Function for dumping all data collected to disk
	 */
	public void save () {
		/* FIXME: make this function use getDefaults().LOG_FILE if it isn't null */
		if (!getSave()) {
			if (getDefaults().bDEBUG)
				System.out.println("StatsPanel: Not saving");
			return;
		}
		if (getDefaults().bDEBUG)
			System.out.println("StatsPanel: Saving Stats");
		Calendar now = Calendar.getInstance();
		for (int i = 0; i < getStats().size(); i++) {
			((Stats)getStats().elementAt(i)).writeStats(now.get(Calendar.YEAR) + "." +
				now.get(Calendar.MONTH) + "." + now.get(Calendar.DAY_OF_MONTH) + "." +
				now.get(Calendar.HOUR) + "." + now.get(Calendar.MINUTE) + "-" +
				i + ".log");
		}
	}// end save ()
	
	public static int currentInputType() {
		if (TwidorConstants.currentState == TwidorConstants.STATE_KEYBOARD)
			return TwidorConstants.INPUT_KEYBOARD;
		else if (TwidorConstants.USE_MULTITAP)
			return TwidorConstants.INPUT_MULTITAP;
		else
			return TwidorConstants.INPUT_TWIDDLER;
	}

}// end class StatsPanel
