/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
 * CCG: Twidor2- The Twiddler Tutor!
 * <pre>
 * Twidor.java, a command-line interface modifying tool.
 *
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 */
public class Twidor {

	public static TwidorConstants twidorDefaults;

	/**
	 * default constructor
	 */
	/*public Twidor () {
		setDefaults(new TwidorConstants());
	}*/// end Twidor ()

	/**
	 * modifier for the defaults
	 * @param TwidorConstants the new defaults
	 */
	private void setDefaults (TwidorConstants defaults) {
		twidorDefaults = defaults;
	}// end setDefaults (TwidorConstants)

	/**
	 * accessor for the defaults
	 * @return TwidorConstants the defaults
	 */
	private static TwidorConstants getDefaults () {
		return twidorDefaults;
	}// end getDefaults ()

	public static boolean parseArgs (String [] argv) {
		int i = 0;
		boolean errors = false;

		/* parse the command line options */
		while (i < argv.length) {
			/* Next string to match */
			String parameter = argv[i++];

			if (parameter.equalsIgnoreCase("-debug")) {
				getDefaults().bDEBUG = true;
			}
				/* MenuBar Options */
			else if (parameter.equalsIgnoreCase("-noMenuBar")) {
				getDefaults().MENUBAR_SHOW = false;
			}
				/* Twiddler Options */
			else if (parameter.equalsIgnoreCase("-noTwiddler")) {
				getDefaults().TWIDDLER_SHOW = false;
				getDefaults().WINDOW_X = 200;
				getDefaults().WINDOW_Y = 300;
			}
			else if (parameter.equalsIgnoreCase("-noTwiddlerLetters")) {
				getDefaults().TWIDDLER_SHOW_LETTERS = false;
			}
			else if (parameter.equalsIgnoreCase("-TwiddlerMirror")) {
				getDefaults().TWIDDLER_MIRROR = true;
			}
				/* Twiddler Highlighting Options */
			else if (parameter.equalsIgnoreCase("-preHighlight")) {
				getDefaults().HIGHLIGHT_HINT = true;
			}
			else if (parameter.equalsIgnoreCase("-postHighlight")) {
				getDefaults().HIGHLIGHT_POST = true;
			}
			else if (parameter.equalsIgnoreCase("-hint")) {
				getDefaults().HIGHLIGHT_HINT = true;
			}
			else if (parameter.equalsIgnoreCase("-hintDelay")) {
				if (i < argv.length) {
					try {
						getDefaults().HIGHLIGHT_DELAY = Integer.parseInt(argv[i++]);
						getDefaults().HIGHLIGHT_HINT = true;
					} catch (Exception e) {
						System.out.println("Not a valid number: " + argv[i - 1]);
					}
				}
				else {
					System.out.println("Option " + parameter +
						" needs an extra parameter");
				}
			}
			else if (parameter.equalsIgnoreCase("-hintMacros")) {
				getDefaults().HIGHLIGHT_MACRO = true;
			}
				/* Error Options */
			else if (parameter.equalsIgnoreCase("-allowErrors")) {
				getDefaults().ALLOW_ERRORS = true;
			}
			else if (parameter.equalsIgnoreCase("-noErrors")) {
				getDefaults().ALLOW_ERRORS = false;
			}
			else if (parameter.equalsIgnoreCase("-highlightErrors")) {
				getDefaults().HIGHLIGHT_ERRORS = true;
			}
			else if (parameter.equalsIgnoreCase("-noHighlightErrors")) {
				getDefaults().HIGHLIGHT_ERRORS = false;
			}
				/* Statistics Options */
			else if (parameter.equalsIgnoreCase("-noStats")) {
				getDefaults().STATS_SHOW = false;
			}
			else if (parameter.equalsIgnoreCase("-Stats")) {
				getDefaults().STATS_SHOW = true;
			}
			else if (parameter.equalsIgnoreCase("-noLog")) {
				getDefaults().LOG_ENABLE = false;
			}
			else if (parameter.equalsIgnoreCase("-Log")) {
				getDefaults().LOG_ENABLE = true;
			}
			else if (parameter.equalsIgnoreCase("-LogFile")) {
				if (i < argv.length) {
					getDefaults().LOG_FILE = argv[i++];
					getDefaults().LOG_ENABLE = true;
				}
				else {
					System.out.println("Option " + parameter +
						" needs an extra parameter");
				}
			}
				/* Sentence Options */
			else if (parameter.equalsIgnoreCase("-SentenceFile")) {
				if (i < argv.length) {
					getDefaults().DEFAULT_PHRASES = argv[i++];
				}
				else {
					System.out.println("Option " + parameter +
						" needs an extra parameter");
				}
			}
				/* KeyMap Options */
			else if (parameter.equalsIgnoreCase("-KeymapFile")) {
				if (i < argv.length) {
					getDefaults().DEFAULT_KEYMAP = argv[i++];
				}
				else {
					System.out.println("Option " + parameter +
						" needs an extra parameter");
				}
			}
				/* Serial Options */
			else if (parameter.equalsIgnoreCase("-serial")) {
				getDefaults().USE_SERIAL = true;
			}
			else if (parameter.equalsIgnoreCase("-serialport")) {
				if (i < argv.length) {
					getDefaults().SERIAL_DEVICE = argv[i++];
					getDefaults().USE_SERIAL = true;
				}
				else {
					System.out.println("Option " + parameter + 
						" needs an extra parameter");
				}
			}
				/* Multitap Options */
			else if (parameter.equalsIgnoreCase("-multitap")) {
				getDefaults().USE_MULTITAP = true;
				getDefaults().DEFAULT_KEYMAP = "multitap.txt";
			}
			else if (parameter.equalsIgnoreCase("-chordingTiming")) {
				getDefaults().TIMING_DATA = true;
				getDefaults().DEFAULT_PHRASES = "timingphrases.txt";
			}
			else if (parameter.equalsIgnoreCase("-multitapTiming")) {
				getDefaults().TIMING_DATA = true;
				getDefaults().DEFAULT_PHRASES = "multitaptimingphrases.txt";
				getDefaults().USE_MULTITAP = true;
				getDefaults().DEFAULT_KEYMAP = "multitap.txt";
			}
			else if (parameter.equalsIgnoreCase("-keyboard")) {
				getDefaults().DO_KEYBOARD = true;
			} else if (parameter.equalsIgnoreCase("-enableMCC")) {
				getDefaults().ALLOW_MCC = true;
			}
			else if (parameter.equalsIgnoreCase("-studyNoMultitap")) {
				getDefaults().STUDY_DO_MULTI_TAP = false;
			}
			else if (parameter.equalsIgnoreCase("-help")) {
				System.out.println("Available options:");
				System.out.println("-debug\t\t\tEnable verbose debugging information.");
				System.out.println("-noMenuBar\t\tDisable the Menubar in the main window.");
				System.out.println("-noTwiddler\t\tDon't show the twiddler at all.");
				System.out.println("-noTwiddlerLetters\tDon't show letters on the face of the twiddler.");
				System.out.println("-TwiddlerMirror\t\tFlip the orientation of the twiddler horizontally.");
				System.out.println("-preHighlight\t\tHighlight the next key to be pressed.");
				System.out.println("-postHighlight\t\tHighlight the key that was pressed.");
				System.out.println("-hint\t\t\tHighlight the next key after some time has passed.");
				System.out.println("-hintDelay\t\tThe length of time (in ms) to wait before highlighting.");
				System.out.println("-hintMacros\t\tEnable macro highlighting (multi-letter chords).");
				System.out.println("-allowErrors\t\tAllow typing to continue even with errors.");
				System.out.println("-noErrors\t\tAllow no typing to continue until errors are fixed.");
				System.out.println("-noStats\t\tDon't show the stats bar.");
				System.out.println("-noLog\t\t\tDon't log any input.");
				System.out.println("-Log\t\t\tLog all input to a file.");
				System.out.println("-LogFile <destination>\tDestination of logged input.");
				System.out.println("-SentenceFile <source>\tUse sentences from this file.");
				System.out.println("-KeyMapFile <source>\tUse a custom keymap.");
				errors = true;
			}
				/* Unknown Options ;D */
			else {
				System.out.println("Unknown option \"" + parameter + "\"");
				errors = true;
			}
		}
		return !errors;
	}

	public static void main (String[] argv) {
		boolean success = parseArgs(argv);

		/* and run the tutor */
		if (success) {
			if (getDefaults().bDEBUG)
				System.out.println("Twidor: Starting TwidorMain");
			TwidorMain tutor = new TwidorMain(getDefaults());
		}
	}// end main

}// end class Twidor
