/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import javax.swing.*;
import java.util.Vector;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * This class is here to automate the Menu switching and flipping
 * and flopping and other fish-like features.
 *
 * Revisions:
 *		0.5	17 July 2003
 *		Completed Tutor
 * 		0.1	17 June 2003
 * 		Created class TwidorMenu
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 17 July 2003
 */
public class TwidorMenu extends JMenuBar {

	/**
	 * internal variables
	 */
	EventHandler myEventHandler;
	TwidorMain myTutor;
	Vector Start, Tutor, Twiddler, Log;
	TwidorConstants myDefaults;

	/**
	 * default constructor
	 */
	private TwidorMenu () {
	}// end TwidorMenu ()

	/**
	 * default constructor
	 */
	public TwidorMenu (TwidorConstants twidorDefaults, TwidorMain tutor, EventHandler events) {

		setDefaults(twidorDefaults);

		JMenu menu;
		JMenuItem jmItem;
		JCheckBoxMenuItem cbItem;
		Tutor(tutor);
		Events(events);

		Start = new Vector();
		menu = new JMenu("Start");
		jmItem = jmenuItem(getDefaults().START_TEXT);
		Start.add(jmItem);
		menu.add(jmItem);
		jmItem = jmenuItem(getDefaults().STOP_TEXT);
		jmItem.setEnabled(false);
		Start.add(jmItem);
		menu.add(jmItem);
		jmItem = jmenuItem(getDefaults().QUIT_TEXT);
		Start.add(jmItem);
		menu.add(jmItem);
		add(menu);

		Tutor = new Vector();
		menu = new JMenu("Tutor");
		jmItem = jcheckItem(getDefaults().HIGHLIGHT_HINT_TEXT,
				getDefaults().HIGHLIGHT_HINT);
		Tutor.add(jmItem);
		menu.add(jmItem);
		jmItem = jcheckItem(getDefaults().HIGHLIGHT_POST_TEXT,
				getDefaults().HIGHLIGHT_POST);
		Tutor.add(jmItem);
		menu.add(jmItem);
		jmItem = jcheckItem(getDefaults().HIGHLIGHT_ERRORS_TEXT,
				getDefaults().HIGHLIGHT_ERRORS);
		Tutor.add(jmItem);
		menu.add(jmItem);
		jmItem = jcheckItem(getDefaults().ALLOW_ERRORS_TEXT,
				getDefaults().ALLOW_ERRORS);
		Tutor.add(jmItem);
		menu.add(jmItem);
		add(menu);

		Twiddler = new Vector();
		menu = new JMenu("Twiddler");
		jmItem = jcheckItem(getDefaults().TWIDDLER_SHOW_TEXT,
				getDefaults().TWIDDLER_SHOW);
		Twiddler.add(jmItem);
		menu.add(jmItem);
		jmItem = jcheckItem(getDefaults().TWIDDLER_MIRROR_TEXT,
				getDefaults().TWIDDLER_MIRROR);
		Twiddler.add(jmItem);
		menu.add(jmItem);
		jmItem = jcheckItem(getDefaults().TWIDDLER_SHOW_LETTERS_TEXT,
				getDefaults().TWIDDLER_SHOW_LETTERS);
		Twiddler.add(jmItem);
		menu.add(jmItem);
		add(menu);

		Log = new Vector();
		menu = new JMenu("Log");
		jmItem = jcheckItem(getDefaults().STATS_SHOW_TEXT,
				getDefaults().STATS_SHOW);
		Log.add(jmItem);
		menu.add(jmItem);
		jmItem = jcheckItem(getDefaults().LOG_ENABLE_TEXT,
				getDefaults().LOG_ENABLE);
		Log.add(jmItem);
		menu.add(jmItem);
		add(menu);

		if (getDefaults().bDEBUG)
			System.out.println("TwidorMenu: Finished Creating");
	}// end TwidorMenu (Twidor, EventHandler)

	/**
	 * modifier for the defaults
	 * @param TwidorConstants the defaults
	 */
	private void setDefaults (TwidorConstants twidorDefaults) {
		myDefaults = twidorDefaults;
	}// end setDefaults (TwidorConstants)

	/**
	 * accessor for the defaults
	 * @return TwidorConstants the defaults
	 */
	private TwidorConstants getDefaults () {
		return myDefaults;
	}// end getDefaults ()

	/**
	 * helper function for creating menu items.
	 * @param String the text of the item
	 * @return JMenuItem the completed item.
	 */
	private JMenuItem jmenuItem (String text) {
		JMenuItem toReturn = new JMenuItem(text);
		toReturn.addActionListener(Events());
		return toReturn;
	}// end jmenuItem (String)

	/**
	 * helper function for creating menu items.
	 * @param String the text of the item
	 * @param boolean the default value of the item
	 * @return JCheckBoxMenuItem the completed item
	 */
	private JCheckBoxMenuItem jcheckItem (String text, boolean status) {
		JCheckBoxMenuItem toReturn = new JCheckBoxMenuItem(text, status);
		toReturn.addItemListener(Events());
		return toReturn;
	}// end jcheckItem (String, boolean)

	/**
	 * Used for ensuring certain elements of the menu can't overlap each other.
	 * @param String the element that just got fiddled with
	 * @param boolean the state it got changed to
	 */
	public void itemSelected (String item, boolean state) {
		if (getDefaults().bDEBUG)
			System.out.println("TwidorMenu: " + item + " selected " + state);
		if (item.equals(getDefaults().START_TEXT)) {
			/* Bad! Do not use constants to access the START and
			 * STOP items */
			((JMenuItem)Start.elementAt(0)).setEnabled(false);
			((JMenuItem)Start.elementAt(1)).setEnabled(true);
		}
		else if (item.equals(getDefaults().STOP_TEXT)) {
			((JMenuItem)Start.elementAt(0)).setEnabled(true);
			((JMenuItem)Start.elementAt(1)).setEnabled(false);
		}
		else if (item.equals(getDefaults().TWIDDLER_SHOW_TEXT)) {
			((JMenuItem)Twiddler.elementAt(1)).setEnabled(state);
			((JMenuItem)Twiddler.elementAt(2)).setEnabled(state);
		}
		else if (item.equals(getDefaults().HIGHLIGHT_HINT_TEXT)) {
			((JMenuItem)Tutor.elementAt(0)).setSelected(state);
			if (state) {
				((JMenuItem)Tutor.elementAt(1)).setSelected(!state);
			}
		}
		else if (item.equals(getDefaults().HIGHLIGHT_POST_TEXT)) {
			((JMenuItem)Tutor.elementAt(1)).setSelected(state);
			if (state) {
				((JMenuItem)Tutor.elementAt(0)).setSelected(!state);
			}
		}
	}// end itemSelected (String, boolean)

	/**
	 * Modifier
	 * @param Twidor
	 */
	private void Tutor (TwidorMain tutor) {
		myTutor = tutor;
	}// end Tutor (Twidor)

	/**
	 * Accessor
	 * @return Twidor
	 */
	public TwidorMain Tutor () {
		return myTutor;
	}// end Tutor ()

	/**
	 * Modifier
	 * @param EventHandler
	 */
	private void Events (EventHandler events) {
		myEventHandler = events;
	}// end Events (EventHandler)

	/**
	 * Accessor
	 * @return EventHandler
	 */
	public EventHandler Events () {
		return myEventHandler;
	}// end Events ()

}// end class TwidorMenu
