/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class TwidorStudy implements TwidorCallbacks
{
	public static TwidorConstants twidorDefaults;

	ArrayList blocks;		//	List of Blocks of 10 sentences each.
	Block curBlock;

	private String message;

	/**
	 * default constructor
	 */
	public TwidorStudy () {
		twidorDefaults = new TwidorConstants();
		blocks = new ArrayList();
		curBlock = new Block();

		twidorDefaults.callbacks = this;
	}// end Twidor ()

	public void runStudy (String [] argv) {
		//	Allow it to be overridden with command line argument
		twidorDefaults.ALLOW_MCC = false;

		boolean success = Twidor.parseArgs(argv);
		if (!success) return;

		success = login();
		if (!success) return;
		
		twidorDefaults.user.sessionNumber++;
		twidorDefaults.user.saveToFile();
		twidorDefaults.user.sessionStarted = System.currentTimeMillis();

		twidorDefaults.MENUBAR_SHOW = false;

		if (twidorDefaults.TIMING_DATA) {
			message = "You will be asked to press a bunch of keys repeatedly, or ";
			message += "to alternate between them.";
		} else if (twidorDefaults.user.sessionNumber == 1 || twidorDefaults.DO_KEYBOARD) {
			twidorDefaults.currentState = twidorDefaults.STATE_KEYBOARD;
			twidorDefaults.ALLOW_KEYBOARD = true;

			message = "First, we'd like to get an idea of your typing speed on ";
			message += "a normal keyboard.  You will be presented with ten ";
			message += "sentences to type.  Type as fast as you can but try ";
			message += "not to make many errors.";
		} else {
			if (!twidorDefaults.user.chording_first && twidorDefaults.STUDY_DO_MULTI_TAP) {
				setMultitap();
			}
			//twidorDefaults.currentState = twidorDefaults.STATE_BLOCK;
			setupWarmup();
		}

		TwidorMain tutor = new TwidorMain(twidorDefaults);
	}
	
	

	public boolean login() {
		BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));

		String username;
		String phone;

		File userfile;
		try {
			while(true) {

				System.out.println("Enter user ID: ");
				username = stdin.readLine();
				System.out.println("Enter phone number: ");
				phone = stdin.readLine();

				userfile = new File(StudyUser.userFile(username));

				if (!userfile.exists()) {
					System.out.print("\nThat user does not exist.  Would you like to create a new one? (y/n) ");
					String yesno = stdin.readLine();
					if (yesno.charAt(0) == 'y') {
						createUser(username, phone);
					} else {
						System.out.println();
						continue;
					}
				}

				StudyUser user = StudyUser.readFromFile(username);
				if (user != null) {
					if (user.phone.equals(phone)) {
						twidorDefaults.user = user;
						twidorDefaults.RUNNING_STUDY = true;
						return true;
					} else {
						System.out.println("Incorrect phone number.\n");
					}
				} else {
					System.out.println("Error reading user file.\n");
				}

			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	}

	public void createUser(String username, String phone) {

		StudyUser user = new StudyUser();
		user.username = username;
		user.phone = phone;
		user.sessionNumber = 0;
		
		//	Decide if the user will do chording or multitap first
		int rand = (int) (Math.random() * 2);
		if (rand == 0) user.chording_first = true;
		else user.chording_first = false;

		user.saveToFile();
	}

	public static String formatint(int num) {
		String ret = "";
		if (num < 10) ret += "0";
		return ret + num;
	}

	public int sentenceDone(Stats sentence) {
		long millisPerSession = 1000 * 60 * 20;
		long elapsedTime = System.currentTimeMillis() - twidorDefaults.user.sessionStarted;
	
		if (sentence != null) {
			curBlock.addSentence(sentence);
			
			//System.out.println("Entered: " + sentence.getEnteredString() + "//END");
            
			//	Get correct directory name and create it if necessary
			String path = twidorDefaults.PATH_TO_DATA + twidorDefaults.user.username;
			if (twidorDefaults.TIMING_DATA) {
				if (!twidorDefaults.USE_MULTITAP) {
					path += File.separator + "ChordingTimingData" + File.separator;
				} else {
					path += File.separator + "MultitapTimingData" + File.separator;
				}
			} else {
				path += File.separator + "Session" + formatint(twidorDefaults.user.sessionNumber);
				path += File.separator;
			}

			if (TwidorConstants.currentState == TwidorConstants.STATE_WARMUP) path += "Warmup";
			else path += "Block";

			path += formatint(blocks.size() + 1);
			File file = new File(path);
			file.mkdirs();
			
			if (twidorDefaults.TIMING_DATA) {
				path += File.separator + "Phrase" + formatint(twidorDefaults.SENTENCE_NUMBER);
			} else {
				path += File.separator + "Sentence" + formatint(curBlock.size());
			}
			
			sentence.writeStats(path);
		} else {
			//	null is passed to this function when the block status screen ends
			//	here we check to see if we go to the next block, switch input types,
			//	or end the session;

			if (twidorDefaults.currentState == twidorDefaults.STATE_ENDED) {
				return twidorDefaults.CODE_END;
			}
			
			if (elapsedTime >= millisPerSession &&
				twidorDefaults.currentState == twidorDefaults.STATE_BLOCK) {

				twidorDefaults.sessionNumber++;
				if ((twidorDefaults.STUDY_DO_MULTI_TAP && twidorDefaults.USE_MULTITAP == twidorDefaults.user.chording_first) ||
					(twidorDefaults.sessionNumber >= 2 && !twidorDefaults.STUDY_DO_MULTI_TAP)) {
					//	Done with this session
					twidorDefaults.currentState = twidorDefaults.STATE_ENDED;
					message = getEndMessage();
					System.out.println(message);
					return twidorDefaults.CODE_NEXT_BLOCK;
				} else {
					System.out.println("Switching input method.");
					if (twidorDefaults.user.chording_first && twidorDefaults.STUDY_DO_MULTI_TAP) {
						setMultitap();
					} else {
						setChording();
					}
					
					//	We need to do the warmup first.
					//twidorDefaults.currentState = twidorDefaults.STATE_WARMUP;
					setupWarmup();
					
					message = getSwitchInputMessage();
					return twidorDefaults.CODE_SWITCH_INPUTS;
				}
			} /*else {
				twidorDefaults.SENTENCE_NUMBER = getSentenceForBlock(curBlock);
				return twidorDefaults.CODE_NEXT_SENTENCE;
			} */
		}

		if (twidorDefaults.TIMING_DATA) {
			if (sentence == null) {
				twidorDefaults.SENTENCE_NUMBER = getSentenceForBlock(curBlock);
				twidorDefaults.timingStarted = 0;

				message = "Type the following sequence of characters as ";
				message += "quickly as you can for the next 10 seconds.\n\n";
				message += twidorDefaults.sentenceList.getSentence(twidorDefaults.SENTENCE_NUMBER);
				return twidorDefaults.CODE_NEXT_SENTENCE;
			} else if (curBlock.size() < twidorDefaults.sentenceList.getSize()) {
				twidorDefaults.timingStarted = 1;
				if (!twidorDefaults.USE_MULTITAP) {
					message = "Done with ten seconds.  Hit the tilde key (~) to go on to the next test.";
				} else {
					message = "Done with ten seconds.  Hit 2, 5, 8, and 0 (at the same time) to go to the next test.";
				}
				return twidorDefaults.CODE_NEXT_BLOCK;
			} else {
				twidorDefaults.timingStarted = 0;
				message = "Done with testing.";
				twidorDefaults.currentState = twidorDefaults.STATE_ENDED;
				return twidorDefaults.CODE_NEXT_BLOCK;
			}
		} else if (twidorDefaults.currentState == twidorDefaults.STATE_WARMUP) {
			//	Warmup
			if (curBlock.size() < 4) {
				if (curBlock.size() % 2 == 0) twidorDefaults.SENTENCE_NUMBER = twidorDefaults.SENTENCE_ALPHA1;
				else twidorDefaults.SENTENCE_NUMBER = twidorDefaults.SENTENCE_ALPHA2;

				return twidorDefaults.CODE_NEXT_SENTENCE;
			} else {
				//	End warmup
				twidorDefaults.currentState = twidorDefaults.STATE_BLOCK;
				twidorDefaults.HIGHLIGHT_HINT = false;
				twidorDefaults.user.sessionStarted = System.currentTimeMillis();

				//	Don't save warmup block for stats
				curBlock = new Block();

				message = "Now that you're warmed up it's time to practice some ";
				message += "\"real\" sentences.  You will have twenty minutes to ";
				message += "complete as many blocks of ten sentences as you can.";
				message += "\nTry to type quickly and accurately- the faster and ";
				message += "more accurately you type, the more money you will be ";
				message += "paid.\n\n";
				message += "Press enter (on the twiddler) to start.";

				return twidorDefaults.CODE_SWITCH_INPUTS;
			}

		} else {
			if (curBlock.size() >= 10 || elapsedTime > (millisPerSession * 1.1)) {
				
				blocks.add(curBlock);
				curBlock = new Block();

				twidorDefaults.SENTENCE_NUMBER = getSentenceForBlock(curBlock);
				
				if (twidorDefaults.currentState == twidorDefaults.STATE_KEYBOARD &&
					blocks.size() > 1) {
					//	done with getting base keyboard typing rate, go to warmup
					if (!twidorDefaults.user.chording_first && twidorDefaults.STUDY_DO_MULTI_TAP) {
						setMultitap();
					}
					setupWarmup();
					return twidorDefaults.CODE_SWITCH_INPUTS;
				} else {
					message = getBlockMessage();	
					return twidorDefaults.CODE_NEXT_BLOCK;
				}

			} else {
				twidorDefaults.SENTENCE_NUMBER = getSentenceForBlock(curBlock);
				return twidorDefaults.CODE_NEXT_SENTENCE;
			}
		}
	}
	
	public void setupWarmup() {
		twidorDefaults.currentState = twidorDefaults.STATE_WARMUP;
		twidorDefaults.ALLOW_KEYBOARD = false;
		twidorDefaults.HIGHLIGHT_HINT = true;
		twidorDefaults.HIGHLIGHT_DELAY = 0;

		message = "You will be using the ";
		if (twidorDefaults.USE_MULTITAP) message += "multi-tap";
		else message += "chording";
		message += " input method for this part of the experiment.\n";
		message += "To get you warmed up, we'll let you type the ";
		message += "alphabet twice.";
		message += "\n\nPress Enter (on the Twiddler) to continue.";
	}


	public String getMessage() {
		return message;
	}

	public String getBlockMessage() {
		Block block = (Block) blocks.get(blocks.size() - 1);

		String msg = "You have finished a block of ten sentences.\n";
		msg += "Typing speed:   " + block.getWPM() + " wpm\n";
		msg += "Accuracy:       " + block.getAccuracy() + "%\n";
		msg += "Adjusted speed: " + (int) (block.getWPM() * block.getAccuracy() / 100.0) + " wpm\n\n";
		msg += "Press enter to continue.";

		return msg;
	}

	public String getSwitchInputMessage() {
		String msg = "You have finished the first half of this session.  ";
		/*
		msg += "In the next part, you will use the ";
		if (twidorDefaults.USE_MULTITAP) msg += "chording";
		else msg += "multi-tap";
		msg += " input method.\n\n";
		*/
		msg += "You may take a five minute break before continuing.  When ";
		msg += "you are ready, press enter.";

		return msg;
	}
	
	public String getEndMessage() {
		double [] stats = getStatsForBlocks(blocks);
		
		double WPM = stats[1];
		double accuracy = stats[2];
		double adjustedWPM = ((int) (WPM * accuracy)) / 100.0;
		String fmtDollars = new DecimalFormat("###.##").format(adjustedWPM);
	
		String msg = "You are finished with this session.\n";
		//msg += "Blocks completed: " + blocks.size() + "\n";
		msg += "Average WPM:      " + WPM + "\n";
		msg += "Average accuracy: " + accuracy + "%\n";
		msg += "Compensation:     $" + adjustedWPM + "\n";
		msg += "User ID:          " + twidorDefaults.user.username + "\n";
		msg += "Thank you for your participation!  Please see the researcher";
		
		
		return msg;
	}

	public void setMultitap() {
		twidorDefaults.USE_MULTITAP = true;
		twidorDefaults.DEFAULT_KEYMAP = "multitap.txt";
		twidorDefaults.TWIDDLER_MIRROR = true;
	}

	public void setChording() {
		twidorDefaults.USE_MULTITAP = false;
		twidorDefaults.DEFAULT_KEYMAP = "thadmap.txt";
		twidorDefaults.TWIDDLER_MIRROR = false;
	}

	public int getSentenceForBlock(Block block) {
		while (true) {
			int num = (int) (Math.random() * twidorDefaults.sentenceList.getSize());
			String newSentence = twidorDefaults.sentenceList.getSentence(num);

			if (!block.sentenceUsed(newSentence) || block.size() >= twidorDefaults.sentenceList.getSize()) {
				//	Warning: if this is used for a large block size and a large number of sentences
				//	in the list, it could be a while before a sentence is found that hasn't been
				//	used yet.
				return num;
			}
		}
	}
	
	public double [] getStatsForBlocks(ArrayList blocks) {
		Vector allStats = new Vector();
		for (int i=0; i<blocks.size(); i++) {
			Block block = (Block) blocks.get(i);
			for (int j=0; j<block.size(); j++) {
				Stats stats = block.getSentence(j);
				allStats.addElement(stats);
			}
		}
		return Stats.getAggregateStats(allStats);
	}

	public static void main (String [] argv) {
		new TwidorStudy().runStudy(argv);
	}
}
