/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.lang.String;
/**
 * CCG: Twidor- The Twiddler Tutor!
 * <pre>
 * TypingPanel.java, because I figured "Everything else has it's own class,
 * why not this one, too!".
 *
 * Revisions:
 *		0.5	17 July 2003
 *		Completed Tutor
 * 		0.1	10 June 2003
 * 		Created class TypingPanel
 * </pre>
 * @author <a href="mailto:visyz@cc.gatech.edu">Gibby Fusia</a>
 * @version Version 0.5; 17 July 2003
 */
public class TypingPanel extends JPanel {

	/**
	 * internal variables
	 */
	Vector Sentence;
	Vector Type;
	int current;
	boolean finished;
	TwidorConstants myDefaults;
	
	boolean showingMessage = false;

	/**
	 * default constructor
	 */
	public TypingPanel (TwidorConstants twidorDefaults) {
		Sentence = new Vector();
		Type = new Vector();

		setDefaults(twidorDefaults);
		getDefaults().CHARACTER_NUMBER = 0;
		setEntered(false);
		setBackground(getDefaults().TEXT_BACKGROUND);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setVisible(true);
		if (getDefaults().bDEBUG)
			System.out.println("TypingPanel: Created New Panel");
	}// end TypingPanel ()

	/**
	 * modifier for the defaults
	 * @param TwidorConstants the new defaults
	 */
	private void setDefaults (TwidorConstants twidorDefaults) {
		myDefaults = twidorDefaults;
	}// end setDefaults (TwidorConstants)

	/**
	 * Private function to determine some whacky stupid things
	 * @param boolean the new value to set
	 */
	private void setEntered (boolean value) {
		finished = value;
	}// end setEntered (boolean)

	/**
	 * Accessor for the defaults
	 * @return TwidorConstants the defaults
	 */
	private TwidorConstants getDefaults () {
		return myDefaults;
	}// end getDefaults ()

	/**
	 * Accessor for finding the character to highlight next
	 * @return char the character to highlight next
	 */
	public String getHighlight () {
		if (getDefaults().CHARACTER_NUMBER < 0) {
			return null;
		}
		else if (getDefaults().CHARACTER_NUMBER >= Sentence.size()) {
			return String.valueOf((char)getDefaults().KEY_ENTER);
		}
		else if (Sentence.size() == 0) {
			return null;
		}
		return ((JLabel)Sentence.elementAt(getDefaults().CHARACTER_NUMBER)).getText();
	}// end getHighlight ()

	/**
	 * Accessor for the finished variable
	 * @return boolean the finished variable
	 */
	private boolean getEntered () {
		return finished;
	}// end getEntered ()

	/**
	 * Called when a new sentence is to be displayed
	 * @param String the new sentence to display
	 */
	public void displaySentence (String sentence) {
		setVisible(false);
		showingMessage = false;
		
		JPanel center = new JPanel();
		center.setAlignmentY(Component.CENTER_ALIGNMENT);

		if (getDefaults().bDEBUG) {
			System.out.println("TypingPanel: Displaying new sentence:");
		}
		//	If the sentence is to long to fit on the screen, it won't display
		//	correctly.  So we print it to the console here just in case.
		System.out.println(sentence);


		/* Empty everything in this JPanel, and clear all associated variables */
		Sentence.clear();
		Type.clear();
		getDefaults().CHARACTER_NUMBER = 0;
		setEntered(false);
		removeAll();

		if (sentence == null) {
			if (getDefaults().bDEBUG)
				System.out.println("TypingPanel: Got sent a null sentence");
			return;
		}
		center.setLayout(new GridLayout(2, sentence.length()));
		center.setBackground(getDefaults().TEXT_BACKGROUND);
		for (int i = 0; i < sentence.length(); i++) {
			JLabel newLetter = new JLabel(String.valueOf(sentence.charAt(i)));
			newLetter.setFont(new Font("Monospaced", Font.PLAIN, 20));
			newLetter.setForeground(getDefaults().TEXT_DEFAULT);
			newLetter.setBackground(getDefaults().TEXT_BACKGROUND);
			Sentence.add(newLetter);
			center.add(newLetter);
		}
		for (int i = 0; i < sentence.length(); i++) {
			JLabel newLetter = new JLabel();
			newLetter.setFont(new Font("Monospaced", Font.PLAIN, 20));
			newLetter.setBackground(getDefaults().TEXT_BACKGROUND);
			if (i == 0) {
				newLetter.setText(getDefaults().CURSOR);
				newLetter.setForeground(getDefaults().TEXT_CURSOR);
			}
			Type.add(newLetter);
			center.add(newLetter);
		}
		center.setMaximumSize(center.getPreferredSize());
		add(Box.createVerticalGlue());
		add(center);
		setVisible(true);
		if (getDefaults().bDEBUG)
			System.out.println("TypingPanel: New sentence displayed");
	}// end displaySentence ()

	public void displayStatus(String message) {
		showingMessage = true;
		setVisible(false);
		setEntered(false);
		removeAll();

		JTextArea label = new JTextArea(message);
		label.setFont(new Font("Monospaced", Font.PLAIN, 18));
		label.setForeground(getDefaults().TEXT_DEFAULT);
		label.setBackground(getDefaults().TEXT_BACKGROUND);
		label.setEditable(false);
		label.setLineWrap(true);
		label.setWrapStyleWord(true);

		add(label);

		setVisible(true);
	}
	
	public void stringTyped(String write) {
		for (int i=0; i<write.length(); i++) {
			char ch = write.charAt(i);
			charTyped((int) ch);
		}
	}

	/**
	 * a new function for writing to the screen
	 * @param int the character to write
	 * @param boolean whether or not to advance
	 */
	public void charTyped (int write) {
		JLabel temp;
		if (getDefaults().CHARACTER_NUMBER < 0) {
			getDefaults().CHARACTER_NUMBER = 0;
		}
		//setVisible(false);
		if (write < 0) {
			if (getDefaults().bDEBUG) 
				System.out.println("TypingPanel: Negative character received?");
		}
		
		else if (write == getDefaults().KEY_BACKSPACE || write == getDefaults().KEY_DELETE ||
				(write == getDefaults().KEY_LEFT && getDefaults().USE_MULTITAP)) { 
			if (getDefaults().bDEBUG)
				System.out.println("TypingPanel: Erasing");
			/* erase! */
			if (getDefaults().CHARACTER_NUMBER >= 0) {
				if (getDefaults().CHARACTER_NUMBER < Type.size()) {
					((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setText("");
					((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)
					 			).setForeground(getDefaults().TEXT_DEFAULT);
					((JLabel)Sentence.elementAt(getDefaults().CHARACTER_NUMBER)
								).setForeground(getDefaults().TEXT_DEFAULT);
				}
				getDefaults().CHARACTER_NUMBER--;
				if (getDefaults().CHARACTER_NUMBER >= 0) {
					((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)
								).setForeground(getDefaults().TEXT_CURSOR);
					((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setText(
								getDefaults().CURSOR);
					((JLabel)Sentence.elementAt(getDefaults().CHARACTER_NUMBER)
								).setForeground(getDefaults().TEXT_DEFAULT);
				}
			}
		}
		else if (write == getDefaults().KEY_ENTER || write == getDefaults().KEY_EOL ||
				(getDefaults().USE_MULTITAP && write == getDefaults().KEY_MENTER)) {
			/* enter! */
			if (getDefaults().bDEBUG)
				System.out.println("TypingPanel: Enter");
			if ((getDefaults().CHARACTER_NUMBER >= Type.size() - getDefaults().HOW_EARLY_ENTER_ALLOWED) ||
					getDefaults().ALLOW_EARLY_ENTER) {
				setEntered(true);
			}
		}
		else if (getDefaults().CHARACTER_NUMBER >= Type.size()) {
			/* quick catchall to prevent oob
			 * nothing beyond this point works if >= Type.size() */
			if (getDefaults().bDEBUG)
				System.out.println("TypingPanel: Ignoring");
		}
		else if (write == getDefaults().KEY_RIGHT && getDefaults().USE_MULTITAP) {
			/* forced advance */
			if (getDefaults().bDEBUG)
				System.out.println("TypingPanel: Advancing");
			((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER++)).setForeground(
								getDefaults().TEXT_DEFAULT);
			if (getDefaults().CHARACTER_NUMBER < Type.size()) {
				((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setForeground(
								getDefaults().TEXT_CURSOR);
				((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setText(
								getDefaults().CURSOR);
			}
		}
		else {
			/* reg'lar key */
			if (getDefaults().bDEBUG)
				System.out.println("TypingPanel: Typing");
			((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setForeground(
						getDefaults().TEXT_DEFAULT);
			((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setText(
						String.valueOf((char)write));
			/* figure out if it was correct */
			String toMatch = ((JLabel)Sentence.elementAt(
						getDefaults().CHARACTER_NUMBER)).getText();
			if (toMatch.equals(String.valueOf((char)write)) ||
				!getDefaults().HIGHLIGHT_ERRORS) {
				((JLabel)Sentence.elementAt(getDefaults().CHARACTER_NUMBER)).setForeground(
								getDefaults().TEXT_GOOD);
			}
			else {
				((JLabel)Sentence.elementAt(getDefaults().CHARACTER_NUMBER)).setForeground(
								getDefaults().TEXT_ERROR);
			}

			if (getDefaults().USE_MULTITAP && write != getDefaults().KEY_SPACE) {
				/* spin */
				((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)).setForeground(
								getDefaults().TEXT_CURSOR);
			}
			else {
				/* advance to next key */
				getDefaults().CHARACTER_NUMBER++;
				if (getDefaults().CHARACTER_NUMBER < Type.size()) {
					((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)
					 			).setText(getDefaults().CURSOR);
					((JLabel)Type.elementAt(getDefaults().CHARACTER_NUMBER)
					 			).setForeground(getDefaults().TEXT_CURSOR);
				}
			}
		}
		//setVisible(true);
	}// end charTyped (int)

	/**
	 * Called to check and see if the sentence is complete
	 */
	public boolean sentenceComplete () {
		if (getDefaults().ALLOW_ERRORS) {
			return getEntered();
		}
		return (getDefaults().CHARACTER_NUMBER >= Sentence.size());
	}// end sentenceComplete ()

}// end class TypingPanel
