/* Twidor - The twiddler tutor (among other things).
 * Copyright (C) 2003 James Fusia
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.util.*;
import java.util.zip.*;

public class UserData implements Serializable
{
	private String username;

	public ArrayList sessions;
	public ArrayList sentences;


	public UserData()
	{
		sessions = new ArrayList();
		sentences = new ArrayList();
	}

	public static UserData read(String username) {
		StudyUser user = StudyUser.readFromFile(username);
		if (user == null) return null;

		StatsBrowser browser = new StatsBrowser();
		browser.username = username;
		
		UserData ret = new UserData();
		ret.username = username;

		String path;

		for (browser.session = 1; browser.session < 50; browser.session++) {
			if (browser.sessionPath() == null) continue;
			ArrayList sessionList = null;

			String filename = browser.sessionPath() + ".dat";
			if (new File(filename).exists()) {
				try {
					ObjectInputStream in = new ObjectInputStream(
						new BufferedInputStream(
						new GZIPInputStream(
						new FileInputStream(filename))));

					sessionList = (ArrayList) in.readObject();
					in.close();
				} catch (Exception e) {
					sessionList = null;
					e.printStackTrace();
				}
			}

			if (sessionList != null) {
				System.out.println("Read " + filename);
			} else {
				sessionList = new ArrayList();
				for (browser.block = 1; browser.blockPath() != null; browser.block++) {
					for (browser.sentence = 1; browser.sentenceFile() != null; browser.sentence++) {
						Stats stats = Stats.readStats(browser.sentenceFile());
						if (stats != null) {
							ret.sentences.add(stats);
							sessionList.add(stats);
							//System.out.println("Read file: " + browser.sentenceFile());
						} else {
							System.out.println("Unable to read file: " + browser.sentenceFile());
						}
					}
				}
				System.out.println("Read session: " + browser.sessionPath());
				ret.sessions.add(sessionList);

				try {
					
					ObjectOutputStream out = new ObjectOutputStream(
						new BufferedOutputStream(
						new GZIPOutputStream(
						new FileOutputStream(filename))));
					out.writeObject(sessionList);
					out.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		}
		System.out.println("Done");

		return ret;
	}
}
